@@ -6,6 +6,7 @@ extern int BOXATTR;
 extern int ACTIVEATTR;
 extern int BARSTDATTR;
 extern int BARHIGHATTR;
+extern int BARPTRATTR;
 extern int DLGTEXTATTR;
 extern int DLGHIGHATTR;
 extern int DLGBOXATTR;
@@ -13,6 +14,7 @@ extern int DESCATTR;
 extern int STATUSBARATTR;
 extern int IPSTATATTR;
 extern int IPSTATLABELATTR;
+extern int DESKTEXTATTR;
 extern int PTRATTR;
 extern int FIELDATTR;
 extern int ERRBOXATTR;
