@@ -21,7 +21,8 @@ int main(void)
     do {
 	direntry = readdir(dir);
 	if (direntry != NULL) {
-	    if (((strlen(direntry->d_name) > 6) && (strncmp(direntry->d_name, "iptraf", 6) == 0))) {
+	    if (((strlen(direntry->d_name) > 6)
+		 && (strncmp(direntry->d_name, "iptraf", 6) == 0))) {
 		cptr = direntry->d_name + 6;
 		strcpy(fullpath1, WORKDIR);
 		strcpy(fullpath2, WORKDIR);
