@@ -21,9 +21,10 @@ details.
 #include <stdlib.h>
 #include <panel.h>
 #include <string.h>
-#include "stdwinset.h"
+#include <labels.h>
+#include <winops.h>
+#include <msgboxes.h>
 #include "deskman.h"
-#include "error.h"
 
 /* Attribute variables */
 
@@ -33,6 +34,7 @@ int BOXATTR;
 int ACTIVEATTR;
 int BARSTDATTR;
 int BARHIGHATTR;
+int BARPTRATTR;
 int DLGTEXTATTR;
 int DLGBOXATTR;
 int DLGHIGHATTR;
@@ -40,6 +42,7 @@ int DESCATTR;
 int STATUSBARATTR;
 int IPSTATLABELATTR;
 int IPSTATATTR;
+int DESKTEXTATTR;
 int PTRATTR;
 int FIELDATTR;
 int ERRBOXATTR;
@@ -79,49 +82,6 @@ void draw_desktop(void)
     refresh();
 }
 
-/* for some reason, the ncurses box routine does not color
-   its inside.  it would therefore be better to call this
-   routine before drawing the box.
- */  
-
-void colorwin(WINDOW * win)
-{
-    int ctr;
-    char *blankpad;
-    blankpad = (char *) malloc(sizeof(char) * (COLS + 1));
-
-    strcpy(blankpad, "");
-
-    for (ctr = 0; ctr <= win->_maxx; ctr++) {
-	strcat(blankpad, " ");
-    }
-
-    scrollok(win, 0);
-    for (ctr = 0; ctr <= win->_maxy; ctr++) {
-	wmove(win, ctr, 0);
-	wprintw(win, "%s", blankpad);
-    }
-    scrollok(win, 1);
-    free(blankpad);
-}
-
-void coloreol()
-{
-    char *blankpad;
-    int y, x;
-    int i;
-
-    blankpad = (char *) malloc(sizeof(char) * (COLS + 1));
-    strcpy(blankpad, "");
-    getyx(stdscr, y, x);
-
-    for (i = x; i <= COLS - 1; i++)
-	strcat(blankpad, " ");
-
-    printw(blankpad);
-    free(blankpad);
-}
-
 void about()
 {
     WINDOW *win;
@@ -131,21 +91,21 @@ void about()
     win = newwin(15, 50, (LINES - 15) / 2, (COLS - 50) / 2);
     panel = new_panel(win);
 
-    stdwinset(win);
+    tx_stdwinset(win);
     wtimeout(win, -1);
     wattrset(win, BOXATTR);
-    colorwin(win);
+    tx_colorwin(win);
     box(win, ACS_VLINE, ACS_HLINE);
     wattrset(win, STDATTR);
     mvwprintw(win, 1, 2, "IPTraf");
     mvwprintw(win, 2, 2, "An IP Network Statistics Utility");
     mvwprintw(win, 3, 2, "Version %s", VERSION);
     mvwprintw(win, 5, 2, "Written by Gerard Paul Java");
-    mvwprintw(win, 6, 2, "Copyright (c) Gerard Paul Java 1997-2001");
-    mvwprintw(win, 8, 2, "This program is distributed under the terms of");
-    mvwprintw(win, 9, 2, "the GNU General Public License Version 2 or");
-    mvwprintw(win, 10, 2, "any later version. See the included COPYING");
-    mvwprintw(win, 11, 2, "file for details.");
+    mvwprintw(win, 6, 2, "Copyright (c) Gerard Paul Java 1997-2002");
+    mvwprintw(win, 8, 2, "This program is open-source software released");
+    mvwprintw(win, 9, 2, "under the terms of the GNU General Public");
+    mvwprintw(win, 10, 2, "Public License Version 2 or any later version.");
+    mvwprintw(win, 11, 2, "See the included LICENSE file for details.");
 
     wattrset(win, HIGHATTR);
     mvwprintw(win, 13, 2, ANYKEY_MSG);
@@ -156,7 +116,7 @@ void about()
     do {
 	ch = wgetch(win);
 	if (ch == 12)
-	    refresh_screen();
+	    tx_refresh_screen();
     } while (ch == 12);
 
     del_panel(panel);
@@ -171,7 +131,7 @@ void show_sort_statwin(WINDOW **statwin, PANEL **panel)
     *panel = new_panel(*statwin);
     
     wattrset(*statwin, BOXATTR);
-    colorwin(*statwin);
+    tx_colorwin(*statwin);
     box(*statwin, ACS_VLINE, ACS_HLINE);
     
     wattrset(*statwin, STDATTR);
@@ -190,56 +150,31 @@ void printipcerr()
     mvprintw(0, 68, "  IPC Error ");
 }
 
-void printkeyhelp(char *keytext, char *desc, WINDOW * win,
-                  int highattr, int textattr)
-{
-    wattrset(win, highattr);
-    wprintw(win, "%s", keytext);
-    wattrset(win, textattr);
-    wprintw(win, "%s", desc);
-}
-
 void stdkeyhelp(WINDOW * win)
 {
-    printkeyhelp("Enter", "-accept  ", win, DLGHIGHATTR, DLGTEXTATTR);
-    printkeyhelp("Ctrl+X", "-cancel", win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("Enter", "-accept  ", win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("Ctrl+X", "-cancel", win, DLGHIGHATTR, DLGTEXTATTR);
 }
 
 void sortkeyhelp(void)
 {
-    printkeyhelp("S", "-sort  ", stdscr, HIGHATTR, STATUSBARATTR);
+    tx_printkeyhelp("S", "-sort  ", stdscr, HIGHATTR, STATUSBARATTR);
 }
 
 void stdexitkeyhelp(void)
 {
-    printkeyhelp("X", "-exit", stdscr, HIGHATTR, STATUSBARATTR);
-    coloreol(stdscr);
+    tx_printkeyhelp("X", "-exit", stdscr, HIGHATTR, STATUSBARATTR);
+    tx_coloreol();
 }
 
 void scrollkeyhelp(void)
 {
-    printkeyhelp("Up/Down/PgUp/PgDn", "-scroll window  ", stdscr, HIGHATTR, STDATTR);
-}
-
-void menukeyhelp(void)
-{
-    move(LINES - 1, 1);
-    printkeyhelp("Up/Down", "-Move selector  ", stdscr, HIGHATTR, STATUSBARATTR);
-    printkeyhelp("Enter", "-execute", stdscr, HIGHATTR, STATUSBARATTR);
-    coloreol(stdscr);
-}
-
-void listkeyhelp()
-{
-    move(LINES - 1, 1);
-    printkeyhelp("Up/Down", "-move pointer  ", stdscr, HIGHATTR, STATUSBARATTR);
-    printkeyhelp("Enter", "-select  ", stdscr, HIGHATTR, STATUSBARATTR);
-    stdexitkeyhelp();
+    tx_printkeyhelp("Up/Down/PgUp/PgDn", "-scroll window  ", stdscr, HIGHATTR, STDATTR);
 }
 
 void tabkeyhelp(WINDOW * win)
 {
-    printkeyhelp("Tab", "-next field  ", win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("Tab", "-next field  ", win, DLGHIGHATTR, DLGTEXTATTR);
 }
 
 void indicate(char *message)
@@ -258,38 +193,14 @@ void printlargenum(unsigned long long i, WINDOW * win)
 	wprintw(win, "%9llu", i);
     else if (i < 1000000000)	/* less than 1 billion */
 	wprintw(win, "%8lluK", i / 1000);
-    else if (i < 100000000000ULL)	/* less than 100 billion */
+    else if (i < 1000000000000ULL)	/* less than 1 trillion */
 	wprintw(win, "%8lluM", i / 1000000);
-    else if (i < 100000000000000ULL)	/* less than 100 trillion */
+    else if (i < 1000000000000000ULL)	/* less than 1000 trillion */
 	wprintw(win, "%8lluG", i / 1000000000ULL);
     else
 	wprintw(win, "%8lluT", i / 1000000000000ULL);
 }
 
-void infobox(char *text, char *prompt)
-{
-    WINDOW *win;
-    PANEL *panel;
-    
-    win = newwin(4, 50, (LINES - 4) / 2, (COLS - 50) / 2);
-    panel = new_panel(win);
-    wattrset(win, BOXATTR);
-    colorwin(win);
-    box(win, ACS_VLINE, ACS_HLINE);
-    wattrset(win, STDATTR);
-    mvwprintw(win, 1, 2, text);
-    wattrset(win, HIGHATTR);
-    mvwprintw(win, 2, 2, prompt);
-    update_panels();
-    doupdate();
-    wgetch(win);
-    del_panel(panel);
-    delwin(win);
-
-    update_panels();
-    doupdate();
-}
-
 void standardcolors(int color)
 {
     if ((color) && (has_colors())) {
@@ -308,13 +219,15 @@ void standardcolors(int color)
 	init_pair(15, COLOR_YELLOW, COLOR_BLACK);
 	init_pair(16, COLOR_WHITE, COLOR_CYAN);
 	init_pair(17, COLOR_YELLOW, COLOR_CYAN);
-
+        init_pair(18, COLOR_GREEN, COLOR_BLACK);
+        
 	STDATTR = COLOR_PAIR(14) | A_BOLD;
 	HIGHATTR = COLOR_PAIR(3) | A_BOLD;
 	BOXATTR = COLOR_PAIR(3);
 	ACTIVEATTR = COLOR_PAIR(10) | A_BOLD;
 	BARSTDATTR = COLOR_PAIR(15) | A_BOLD;
 	BARHIGHATTR = COLOR_PAIR(11) | A_BOLD;
+	BARPTRATTR = COLOR_PAIR(18) | A_BOLD;
 	DESCATTR = COLOR_PAIR(2);
 	DLGTEXTATTR = COLOR_PAIR(2);
 	DLGBOXATTR = COLOR_PAIR(6);
@@ -322,6 +235,7 @@ void standardcolors(int color)
 	STATUSBARATTR = STDATTR;
 	IPSTATLABELATTR = COLOR_PAIR(2);
 	IPSTATATTR = COLOR_PAIR(12);
+	DESKTEXTATTR = COLOR_PAIR(7);
 	PTRATTR = COLOR_PAIR(10) | A_BOLD;
 	FIELDATTR = COLOR_PAIR(1);
 	ERRBOXATTR = COLOR_PAIR(5) | A_BOLD;
@@ -335,20 +249,22 @@ void standardcolors(int color)
 	GREATTR = COLOR_PAIR(1);
 	UNKNATTR = COLOR_PAIR(4) | A_BOLD;
     } else {
-	STDATTR = A_NORMAL;
-	HIGHATTR = A_BOLD;
-	BOXATTR = A_NORMAL;
+	STDATTR = A_REVERSE;
+	HIGHATTR = A_REVERSE;
+	BOXATTR = A_REVERSE;
 	ACTIVEATTR = A_BOLD;
-	BARSTDATTR = A_REVERSE;
+	BARSTDATTR = A_NORMAL;
 	BARHIGHATTR = A_BOLD;
-	DESCATTR = A_NORMAL;
+	BARPTRATTR = A_NORMAL;
+	DESCATTR = A_BOLD;
         DLGBOXATTR = A_REVERSE;
         DLGTEXTATTR = A_REVERSE;
         DLGHIGHATTR = A_BOLD;
         STATUSBARATTR = A_REVERSE;
         IPSTATLABELATTR = A_REVERSE;
         IPSTATATTR = A_STANDOUT;
-	PTRATTR = A_BOLD;
+        DESKTEXTATTR = A_NORMAL;
+	PTRATTR = A_REVERSE;
 	FIELDATTR = A_BOLD;
 	ERRBOXATTR = A_BOLD;
 	ERRTXTATTR = A_NORMAL;
@@ -361,11 +277,8 @@ void standardcolors(int color)
 	GREATTR = A_BOLD;
 	UNKNATTR = A_BOLD;
     }
+    
+    tx_init_error_attrs(ERRBOXATTR, ERRTXTATTR, ERRBOXATTR);
+    tx_init_info_attrs(BOXATTR, STDATTR, HIGHATTR);
 }
 
-void refresh_screen(void)
-{
-    endwin();
-    doupdate();
-    curs_set(0);
-}
