@@ -6,7 +6,6 @@
  */
 
 void draw_desktop(void);
-void colorwin(WINDOW * win);
 void printnomem();
 void printipcerr();
 void printkeyhelp(char *keytext, char *desc, WINDOW * win,
@@ -16,11 +15,8 @@ void sortkeyhelp(void);
 void tabkeyhelp(WINDOW * win);
 void scrollkeyhelp();
 void stdexitkeyhelp();
-void menukeyhelp();
-void listkeyhelp();
 void indicate(char *message);
 void printlargenum(unsigned long long i, WINDOW *win);
 void infobox(char *text, char *prompt);
 void standardcolors(int color);
-void refresh_screen(void);
 void show_sort_statwin();
