@@ -7,12 +7,12 @@ others
 ***/
 
 void ipmon(const struct OPTIONS *options, struct filterstate *ofilter,
-           unsigned int tm, int facilitytime, char *ifptr);
+           int facilitytime, char *ifptr);
 void selectiface(char *ifname, int withall, int *aborted);
-void ifstats(const struct OPTIONS *options, int facilitytime);
-void detstats(char *iface, const struct OPTIONS *options, int facilitytime);
+void ifstats(const struct OPTIONS *options, struct filterstate *ofilter, int facilitytime);
+void detstats(char *iface, const struct OPTIONS *options, int facilitytime, struct filterstate *ofilter);
 void packet_size_breakdown(struct OPTIONS *options, char *iface, int facilitytime);
-void servmon(char *iface, struct porttab *ports, const struct OPTIONS *options, int facilitytime);
+void servmon(char *iface, struct porttab *ports, const struct OPTIONS *options, int facilitytime, struct filterstate *ofilter);
 void ip_host_breakdown(struct OPTIONS *options, char *iface);
 void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr);
 void ethdescmgr(void);
