@@ -30,8 +30,10 @@ details.
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <time.h>  
-#include "menurt.h"
-#include "stdwinset.h"
+#include <menurt.h>
+#include <winops.h>
+#include <labels.h>
+#include <msgboxes.h>
 #include "fltdefs.h"
 #include "fltmgr.h"
 #include "othipflt.h"
@@ -87,7 +89,7 @@ void definefilter(int protocol, int *aborted)
 	open(get_path(T_WORKDIR, fntemp), O_CREAT | O_WRONLY | O_TRUNC,
 	     S_IRUSR | S_IWUSR);
     if (pfd < 0) {
-	errbox("Cannot create filter data file", ANYKEY_MSG,
+	tx_errbox("Cannot create filter data file", ANYKEY_MSG,
 	       &resp);
 	*aborted = 1;
 	clear_flt_tag(protocol);
@@ -109,7 +111,7 @@ void definefilter(int protocol, int *aborted)
 	        bw = write(pfd, &(fe.un.othipdata), sizeof(struct othipfltdata));
 
 	    if (bw < 0) {
-		errbox("Unable to write filter data", ANYKEY_MSG,
+		tx_errbox("Unable to write filter data", ANYKEY_MSG,
 		       &resp);
 		close(pfd);
 		clear_flt_tag(protocol);
@@ -239,7 +241,7 @@ void savefilter(unsigned int protocol, char *filename,
 	    bw = write(pfd, &(fe->un.othipdata), sizeof(struct othipfltdata));
 	
 	if (bw < 0) {
-	    errbox("Unable to save filter changes", ANYKEY_MSG, &resp);
+	    tx_errbox("Unable to save filter changes", ANYKEY_MSG, &resp);
 	    clear_flt_tag(protocol);
 	    return;
 	}
@@ -273,10 +275,13 @@ void destroyfilter(struct filterlist *fl)
     }
 }
 
-void print_hostparam_line(int protocol, struct filterent *fe, int idx, WINDOW * win)
+void print_hostparam_line(int protocol, struct filterent *fe, int idx, WINDOW * win, 
+                          int attr)
 {
-    wattrset(win, STDATTR);
-    mvwprintw(win, idx, 2, "%76c", ' ');
+    wattrset(win, attr);
+
+    scrollok(win, 0);
+    mvwprintw(win, idx, 0, "%78c", ' ');
 
     if (protocol != F_OTHERIP) {
         mvwaddnstr(win, idx, 2, fe->un.hp.s_fqdn, 15);
@@ -295,7 +300,7 @@ void print_hostparam_line(int protocol, struct filterent *fe, int idx, WINDOW *
         mvwaddnstr(win, idx, 38, fe->un.othipdata.hp.d_fqdn, 15);
         wprintw(win, "/");
         waddnstr(win, fe->un.othipdata.hp.d_mask, 15);
-        mvwprintw(win, idx, 75, "%c", toupper(fe->un.othipdata.hp.reverse));
+        mvwprintw(win, idx, 76, "%c", toupper(fe->un.othipdata.hp.reverse));
     }
 }
 
@@ -305,16 +310,16 @@ void update_hp_screen(int protocol, struct filterlist *fl,
     struct filterent *ftmp = firstvisible;
     int i;
 
-    scrollok(win, 0);
     wattrset(win, STDATTR);
     if (firstvisible == NULL) {
 	mvwprintw(win, 0, 0, "%78c", ' ');
 	return;
     }
 
+    scrollok(win, 0);
     for (i = 0; i <= 12; i++) {
 	if (ftmp != NULL) {
-	    print_hostparam_line(protocol, ftmp, i, win);
+	    print_hostparam_line(protocol, ftmp, i, win, STDATTR);
 	    ftmp = ftmp->next_entry;
 	} else {
 	    mvwprintw(win, i, 0, "%78c", ' ');
@@ -328,7 +333,7 @@ int new_hp_entry(struct filterent **ftemp)
     int resp;
     *ftemp = malloc(sizeof(struct filterent));
     if (*ftemp == NULL) {
-	errbox("No memory for new filter entry", ANYKEY_MSG, &resp);
+	tx_errbox("No memory for new filter entry", ANYKEY_MSG, &resp);
 	return 0;
     }
     
@@ -382,6 +387,7 @@ void modify_host_parameters(unsigned int protocol, struct filterlist *fl)
     char s_portstr[8];
     char d_portstr[8];
     char inexstr[2];
+    char sp_buf[10];
     union hp_un hptemp;
 
     bwin = newwin(15, 80, (LINES - 15) / 2, (COLS - 80) / 2);
@@ -391,10 +397,12 @@ void modify_host_parameters(unsigned int protocol, struct filterlist *fl)
     panel = new_panel(win);
     
     if (protocol == F_OTHERIP) {
-        swin = newwin(1, 80, (LINES - 15) / 2 + 15, (COLS - 80) / 2);
+        swin = newwin(1, COLS, (LINES - 15) / 2 + 15, 0);
         spanel = new_panel(swin);
         wattrset(swin, DLGTEXTATTR);
         scrollok(swin, 0);
+        sprintf(sp_buf, "%%%dc", COLS);
+        mvwprintw(swin, 0, 0, sp_buf, ' ');
     }
     
     wattrset(bwin, BOXATTR);
@@ -420,17 +428,18 @@ void modify_host_parameters(unsigned int protocol, struct filterlist *fl)
         wprintw(bwin, " Misc IP");
 
     wprintw(bwin, " Filter Data ");
-    stdwinset(win);
+    tx_stdwinset(win);
+    scrollok(win, 0);
     wattrset(win, STDATTR);
-    colorwin(win);
+    tx_colorwin(win);
 
     move(LINES - 1, 1);
-    printkeyhelp("Up/Down", "-move ptr ", stdscr, HIGHATTR, STATUSBARATTR);
-    printkeyhelp("I", "-insert ", stdscr, HIGHATTR, STATUSBARATTR);
-    printkeyhelp("A", "-add to list ", stdscr, HIGHATTR, STATUSBARATTR);
-    printkeyhelp("D", "-delete ", stdscr, HIGHATTR, STATUSBARATTR);
-    printkeyhelp("Enter", "-edit ", stdscr, HIGHATTR, STATUSBARATTR);
-    printkeyhelp("X/Ctrl+X", "-exit", stdscr, HIGHATTR, STATUSBARATTR);
+    tx_printkeyhelp("Up/Down", "-move ptr ", stdscr, HIGHATTR, STATUSBARATTR);
+    tx_printkeyhelp("I", "-insert ", stdscr, HIGHATTR, STATUSBARATTR);
+    tx_printkeyhelp("A", "-add to list ", stdscr, HIGHATTR, STATUSBARATTR);
+    tx_printkeyhelp("D", "-delete ", stdscr, HIGHATTR, STATUSBARATTR);
+    tx_printkeyhelp("Enter", "-edit ", stdscr, HIGHATTR, STATUSBARATTR);
+    tx_printkeyhelp("X/Ctrl+X", "-exit", stdscr, HIGHATTR, STATUSBARATTR);
 
     update_panels();
     doupdate();
@@ -446,9 +455,8 @@ void modify_host_parameters(unsigned int protocol, struct filterlist *fl)
     doupdate();
     
     do {
-	wattrset(win, PTRATTR);
 	if (fe != NULL) {
-	    mvwaddch(win, idx, 1, ACS_RARROW);
+            print_hostparam_line(protocol, fe, idx, win, BARSTDATTR);
 	    if (protocol == F_OTHERIP) {
 	        show_matched_protocols(swin, fe);
 	        update_panels();
@@ -457,7 +465,9 @@ void modify_host_parameters(unsigned int protocol, struct filterlist *fl)
 	}
 	
 	ch = wgetch(win);
-	mvwprintw(win, idx, 1, " ");
+
+        if (fe != NULL)
+            print_hostparam_line(protocol, fe, idx, win, STDATTR);
 
 	switch (ch) {
 	case KEY_UP:
@@ -466,11 +476,8 @@ void modify_host_parameters(unsigned int protocol, struct filterlist *fl)
 		    if (idx > 0)
 			idx--;
 		    else {
+		        scrollok(win, 1);
 			wscrl(win, -1);
-			scrollok(win, 0);
-			mvwprintw(win, 0, 0, "%78c", ' ');
-			print_hostparam_line(protocol, fe->prev_entry, idx, win);
-			scrollok(win, 1);
 			firstvisible = firstvisible->prev_entry;
 		    }
 		    fe = fe->prev_entry;
@@ -483,11 +490,8 @@ void modify_host_parameters(unsigned int protocol, struct filterlist *fl)
 		    if (idx < 12)
 			idx++;
 		    else {
+		        scrollok(win, 1);
 			wscrl(win, 1);
-			scrollok(win, 0);
-			mvwprintw(win, 12, 0, "%78c", ' ');
-			print_hostparam_line(protocol, fe->next_entry, idx, win);
-			scrollok(win, 1);
 			firstvisible = firstvisible->next_entry;
 		    }
 		    fe = fe->next_entry;
@@ -624,9 +628,6 @@ void modify_host_parameters(unsigned int protocol, struct filterlist *fl)
 			 * previous to the target.
 			 */
 			if (ftemp != NULL) {
-			    wscrl(win, -1);
-			    mvwprintw(win, 0, 0, "%78c", ' ');
-			    print_hostparam_line(protocol, ftemp, idx, win);
 			    firstvisible = ftemp;
 			}
 		    }
@@ -681,7 +682,7 @@ void modify_host_parameters(unsigned int protocol, struct filterlist *fl)
 	    break;
 	case 'l':
 	case 'L':
-	    refresh_screen();
+	    tx_refresh_screen();
 	    break;
 	}
 	update_panels();
