@@ -28,62 +28,71 @@ details.
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <listbox.h>
+#include <labels.h>
+#include <input.h>
+#include <menurt.h>
+#include <msgboxes.h>
 #include "attrs.h"
-#include "error.h"
 #include "deskman.h"
-#include "input.h"
-#include "menurt.h"
 #include "dirs.h"
 #include "fltdefs.h"
 #include "fltmgr.h"
 #include "instances.h"
+#include "error.h"
 
 extern int daemonized;
 
 void makestdfiltermenu(struct MENU *menu)
 {
-    initmenu(menu, 8, 31, (LINES - 8) / 2, (COLS - 31) / 2 + 15);
-    additem(menu, " ^D^efine new filter...",
+    tx_initmenu(menu, 9, 31, (LINES - 8) / 2, (COLS - 31) / 2 + 15,
+        BOXATTR, STDATTR, HIGHATTR, BARSTDATTR, BARHIGHATTR, DESCATTR);
+    tx_additem(menu, " ^D^efine new filter...",
 	    "Defines a new set of filter parameters");
-    additem(menu, " ^A^pply filter...",
+    tx_additem(menu, " ^A^pply filter...",
 	    "Applies a defined filter");
-    additem(menu, " Detac^h^ filter",
+    tx_additem(menu, " Detac^h^ filter",
 	    "Removes the currently applied filter");
-    additem(menu, " ^E^dit filter...",
+    tx_additem(menu, " ^E^dit filter...",
 	    "Modifies existing filter data");
-    additem(menu, " Dele^t^e filter...",
+    tx_additem(menu, " Dele^t^e filter...",
 	    "Removes a TCP filter from the filter list");
-    additem(menu, " E^x^it menu", "Returns to the main menu");
+    tx_additem(menu, NULL, NULL);
+    tx_additem(menu, " E^x^it menu", "Returns to the main menu");
 }
 
 void makeudpfiltermenu(struct MENU *menu)
 {
-    initmenu(menu, 9, 38, (LINES - 11) / 2 + 1, (COLS - 38) / 2 + 18);
-    additem(menu, " Show ^a^ll UDP packets",
+    tx_initmenu(menu, 10, 38, (LINES - 11) / 2 + 1, (COLS - 38) / 2 + 18,
+        BOXATTR, STDATTR, HIGHATTR, BARSTDATTR, BARHIGHATTR, DESCATTR);
+    tx_additem(menu, " Show ^a^ll UDP packets",
 	    "Permits display of all UDP packets");
-    additem(menu, " Show ^n^o UDP packets",
+    tx_additem(menu, " Show ^n^o UDP packets",
 	    "Omits all UDP packets from display");
-    additem(menu, " ^D^efine custom UDP filter...",
+    tx_additem(menu, " ^D^efine custom UDP filter...",
 	    "Defines a custom UDP filter");
-    additem(menu, " Apply ^c^ustom UDP filter...",
+    tx_additem(menu, " Apply ^c^ustom UDP filter...",
 	    "Applies a custom UDP filter");
-    additem(menu, " ^E^dit custom UDP filter...",
+    tx_additem(menu, " ^E^dit custom UDP filter...",
 	    "Modifies a custom UDP filter");
-    additem(menu, " Dele^t^e custom UDP filter...",
+    tx_additem(menu, " Dele^t^e custom UDP filter...",
 	    "Removes a custom UDP filter");
-    additem(menu, " E^x^it menu", "Returns to the main menu");
+    tx_additem(menu, NULL, NULL);
+    tx_additem(menu, " E^x^it menu", "Returns to the main menu");
 }
 
 void makemainfiltermenu(struct MENU *menu)
 {
-    initmenu(menu, 9, 15, (LINES - 8) / 2, (COLS - 15) / 2 - 5);
-    additem(menu, " ^T^CP...", "Transmission Control Protocol");
-    additem(menu, " ^U^DP...", "User Datagram Protocol");
-    additem(menu, " ^O^ther IP...", "Other IP protocols");
-    additem(menu, " ^A^RP", "Address Resolution Protocol");
-    additem(menu, " RAR^P^ ", "Reverse Address Resolution Protocol");
-    additem(menu, " ^N^on-IP", "Non-IP packets");
-    additem(menu, " E^x^it menu", "Return to main menu");
+    tx_initmenu(menu, 10, 15, (LINES - 8) / 2, (COLS - 15) / 2 - 5,
+        BOXATTR, STDATTR, HIGHATTR, BARSTDATTR, BARHIGHATTR, DESCATTR);
+    tx_additem(menu, " ^T^CP...", "Transmission Control Protocol");
+    tx_additem(menu, " ^U^DP...", "User Datagram Protocol");
+    tx_additem(menu, " ^O^ther IP...", "Other IP protocols");
+    tx_additem(menu, " ^A^RP", "Address Resolution Protocol");
+    tx_additem(menu, " RAR^P^ ", "Reverse Address Resolution Protocol");
+    tx_additem(menu, " ^N^on-IP", "Non-IP packets");
+    tx_additem(menu, NULL, NULL);
+    tx_additem(menu, " E^x^it menu", "Return to main menu");
 }
 
 /*
@@ -103,7 +112,7 @@ int mark_filter_change(int protocol)
 	if (!facility_active(TCPFLTIDFILE, ""))
 	    mark_facility(TCPFLTIDFILE, "TCP filter change", "");
 	else {
-	    errbox("TCP filter data file in use; try again later",
+	    tx_errbox("TCP filter data file in use; try again later",
 		   ANYKEY_MSG, &resp);
 	    return 0;
 	}
@@ -111,7 +120,7 @@ int mark_filter_change(int protocol)
 	if (!facility_active(UDPFLTIDFILE, ""))
 	    mark_facility(UDPFLTIDFILE, "UDP filter change", "");
 	else {
-	    errbox("UDP filter data file in use; try again later",
+	    tx_errbox("UDP filter data file in use; try again later",
 		   ANYKEY_MSG, &resp);
 	    return 0;
 	}
@@ -119,7 +128,7 @@ int mark_filter_change(int protocol)
         if (!facility_active(OTHIPFLTIDFILE, ""))
             mark_facility(OTHIPFLTIDFILE, "Other IP filter change", "");
         else {
-            errbox("Other IP protocol filter data file in use; try again later",
+            tx_errbox("Other IP protocol filter data file in use; try again later",
                    ANYKEY_MSG, &resp);
             return 0;
         }
@@ -285,130 +294,44 @@ void save_filterlist(unsigned int protocol, struct ffnode *fltlist)
     close(fd);
 }
 
-void displayfilters(struct ffnode *ffiles, WINDOW * win)
-{
-    unsigned int row = 0;
-    struct ffnode *pptr;
-
-    pptr = ffiles;
-
-    wattrset(win, STDATTR);
-    do {
-	wmove(win, row, 2);
-	wprintw(win, pptr->ffe.desc);
-	row++;
-	pptr = pptr->next_entry;
-    } while ((row < 8) && (pptr != NULL));
-
-    update_panels();
-    doupdate();
-}
-
-void operate_select(struct ffnode *ffiles, WINDOW * win,
+void operate_select(struct ffnode *ffiles,
 		    struct ffnode **item, int *aborted)
 {
-    unsigned int row = 0;
     struct ffnode *pptr;
     int ch;
-    int exitloop = 0;
-
-    listkeyhelp();
+    struct scroll_list list;
+    
+    tx_listkeyhelp(STDATTR, HIGHATTR);
     update_panels();
     doupdate();
+    
     pptr = ffiles;
-    do {
-	wattrset(win, PTRATTR);
-	wmove(win, row, 1);
-	waddch(win, ACS_RARROW);
-	ch = wgetch(win);
-	wmove(win, row, 1);
-	wprintw(win, " ");
-	wattrset(win, STDATTR);
-
-	switch (ch) {
-	case KEY_UP:
-	    if (pptr->prev_entry != NULL) {
-		if (row > 0)
-		    row--;
-		else {
-		    wscrl(win, -1);
-		    wmove(win, 0, 0);
-		    wprintw(win, "%58c", ' ');
-		    wmove(win, 0, 2);
-		    wprintw(win, pptr->prev_entry->ffe.desc);
-		}
-		pptr = pptr->prev_entry;
-	    }
-	    break;
-	case KEY_DOWN:
-	    if (pptr->next_entry != NULL) {
-		if (row < 7)
-		    row++;
-		else {
-		    wscrl(win, 1);
-		    scrollok(win, 0);
-		    wmove(win, 7, 0);
-		    wprintw(win, "%58c", ' ');
-		    scrollok(win, 1);
-		    wmove(win, 7, 2);
-		    wprintw(win, pptr->next_entry->ffe.desc);
-		}
-		pptr = pptr->next_entry;
-	    }
-	    break;
-	case 12:
-	case 'l':
-	case 'L':
-	    refresh_screen();
-	    break;
-	case 13:
-	    exitloop = 1;
-	    *aborted = 0;
-	    break;
-	case 27:
-	case 24:
-	case 'X':
-	case 'x':
-	case 'Q':
-	case 'q':
-	    exitloop = 1;
-	    *aborted = 1;
-	    break;
-	}
-	update_panels();
-	doupdate();
-    } while (!exitloop);
-
-    *item = pptr;
+    
+    tx_init_listbox(&list, 60, 10, (COLS - 60) / 2 - 2, (LINES - 10) / 2 - 2,
+        STDATTR, BOXATTR, BARSTDATTR, HIGHATTR);
+    
+    tx_set_listbox_title(&list, "Select Filter", 1);
+    
+    while (pptr != NULL) {
+        tx_add_list_entry(&list, (char *) pptr, pptr->ffe.desc);
+        pptr = pptr->next_entry;
+    }
+    
+    tx_show_listbox(&list);
+    tx_operate_listbox(&list, &ch, aborted);
+    
+    if (!(*aborted))     
+        *item = (struct ffnode *) list.textptr->nodeptr;
+        
+    tx_close_listbox(&list);
+    tx_destroy_list(&list);
 }
 
 void pickafilter(struct ffnode *ffiles,
 		 struct ffnode **fltfile, int *aborted)
 {
-    WINDOW *borderwin;
-    PANEL *borderpanel;
-    WINDOW *selectwin;
-    PANEL *selectpanel;
+    operate_select(ffiles, fltfile, aborted);
 
-    borderwin = newwin(10, 60, (LINES - 10) / 2 - 2, (COLS - 60) / 2 - 2);
-    borderpanel = new_panel(borderwin);
-    wattrset(borderwin, BOXATTR);
-    box(borderwin, ACS_VLINE, ACS_HLINE);
-
-    selectwin = newwin(8, 58, (LINES - 8) / 2 - 2, (COLS - 58) / 2 - 2);
-    selectpanel = new_panel(selectwin);
-    wattrset(selectwin, STDATTR);
-    colorwin(selectwin);
-    stdwinset(selectwin);
-    wtimeout(selectwin, -1);
-
-    displayfilters(ffiles, selectwin);
-    operate_select(ffiles, selectwin, fltfile, aborted);
-
-    del_panel(borderpanel);
-    delwin(borderwin);
-    del_panel(selectpanel);
-    delwin(selectwin);
     update_panels();
     doupdate();
 }
@@ -447,7 +370,7 @@ void get_filter_description(char *description, int *aborted,
     dlgwin = newwin(7, 42, dlgwintop, (COLS - 42) / 2 - 10);
     dlgpanel = new_panel(dlgwin);
     wattrset(dlgwin, DLGBOXATTR);
-    colorwin(dlgwin);
+    tx_colorwin(dlgwin);
     box(dlgwin, ACS_VLINE, ACS_HLINE);
     wattrset(dlgwin, DLGTEXTATTR);
     wmove(dlgwin, 2, 2);
@@ -457,20 +380,21 @@ void get_filter_description(char *description, int *aborted,
     update_panels();
     doupdate();
 
-    initfields(&descfield, 1, 35, dlgwintop + 3, (COLS - 42) / 2 - 8);
-    addfield(&descfield, 33, 0, 0, pre_edit);
+    tx_initfields(&descfield, 1, 35, dlgwintop + 3, (COLS - 42) / 2 - 8,
+        DLGTEXTATTR, FIELDATTR);
+    tx_addfield(&descfield, 33, 0, 0, pre_edit);
     
     do {
-        fillfields(&descfield, aborted);
+        tx_fillfields(&descfield, aborted);
     
         if ((descfield.list->buf[0] == '\0') && (!(*aborted)))
-            errbox("Enter an appropriate description for this filter", ANYKEY_MSG, &resp);
+            tx_errbox("Enter an appropriate description for this filter", ANYKEY_MSG, &resp);
     } while ((descfield.list->buf[0] == '\0') && (!(*aborted)));
     
     if (!(*aborted))
         strcpy(description, descfield.list->buf);
 
-    destroyfields(&descfield);
+    tx_destroyfields(&descfield);
     del_panel(dlgpanel);
     delwin(dlgwin);
     update_panels();
