@@ -18,7 +18,7 @@
 char *get_path(int dirtype, char *file)
 {
     static char path[PATH_MAX];
-    char *ptr;
+    char *ptr = NULL;
     char *dir, *env = NULL;
 
     switch (dirtype) {
@@ -33,11 +33,15 @@ char *get_path(int dirtype, char *file)
     case T_EXECDIR:
 	dir = EXECDIR;
 	break;
+    case T_LOCKDIR:
+        dir = LOCKDIR;
+        break;
     default:
 	return file;
     }
 
-    if ((dirtype != T_EXECDIR) && (ptr = getenv(env)) != NULL)
+    if ((dirtype != T_EXECDIR) && (dirtype != T_LOCKDIR) &&
+       (ptr = getenv(env)) != NULL)
 	dir = ptr;
 
     if (dir == NULL || *dir == '\0')
