@@ -31,27 +31,33 @@ details.
 #include <linux/if_packet.h>
 #include <linux/if_ether.h>
 #include <linux/if_fddi.h>
+#include <linux/if_tr.h>
 #include <net/if_arp.h>
 #include <stdlib.h>
+#include <time.h>
 #include <sys/time.h>
+#include <winops.h>
+#include <labels.h>
 #include "dirs.h"
 #include "deskman.h"
 #include "links.h"
+#include "fltdefs.h"
+#include "utfilter.h"
+#include "fltselect.h"
 #include "isdntab.h"		/* needed by packet.h */
 #include "packet.h"
 #include "ifaces.h"
 #include "hostmon.h"
-#include "stdwinset.h"
 #include "attrs.h"
 #include "log.h"
 #include "timer.h"
 #include "landesc.h"
 #include "options.h"
 #include "instances.h"
-#include "error.h"
 #include "mode.h"
 #include "logvars.h"
 #include "promisc.h"
+#include "error.h"
 
 #define SCROLLUP 0
 #define SCROLLDOWN 1
@@ -133,8 +139,8 @@ void initethtab(struct ethtab *table, int unit)
 	    unitstring);
 
     wattrset(table->tabwin, STDATTR);
-    colorwin(table->tabwin);
-    stdwinset(table->tabwin);
+    tx_colorwin(table->tabwin);
+    tx_stdwinset(table->tabwin);
     wtimeout(table->tabwin, -1);
 
     update_panels();
@@ -400,6 +406,22 @@ void updateethrates(struct ethtab *table, int unit, time_t starttime,
     }
 }
 
+void refresh_hostmon_screen(struct ethtab *table, int idx)
+{
+    struct ethtabent *ptmp = table->firstvisible;
+
+    wattrset(table->tabwin, STDATTR);
+    tx_colorwin(table->tabwin);
+
+    while ((ptmp != NULL) && (ptmp->prev_entry != table->lastvisible)) {
+        printethent(table, ptmp, idx);
+        ptmp = ptmp->next_entry;
+    }
+
+    update_panels();
+    doupdate();
+}
+
 void scrollethwin(struct ethtab *table, int direction, int *idx)
 {
     char sp_buf[10];
@@ -441,14 +463,19 @@ void pageethwin(struct ethtab *table, int direction, int *idx)
     if (direction == SCROLLUP) {
 	while ((i <= LINES - 7) && (table->lastvisible != table->tail)) {
 	    i++;
-	    scrollethwin(table, direction, idx);
+            table->firstvisible = table->firstvisible->next_entry;
+            table->lastvisible = table->lastvisible->next_entry;
+            (*idx)++;
 	}
     } else {
 	while ((i <= LINES - 7) && (table->firstvisible != table->head)) {
 	    i++;
-	    scrollethwin(table, direction, idx);
+	    table->firstvisible = table->firstvisible->prev_entry;
+	    table->lastvisible = table->lastvisible->prev_entry;
+	    (*idx)--;
 	}
     }
+    refresh_hostmon_screen(table, *idx);
 }
 
 void show_hostsort_keywin(WINDOW ** win, PANEL ** panel)
@@ -457,25 +484,25 @@ void show_hostsort_keywin(WINDOW ** win, PANEL ** panel)
     *panel = new_panel(*win);
 
     wattrset(*win, DLGBOXATTR);
-    colorwin(*win);
+    tx_colorwin(*win);
     box(*win, ACS_VLINE, ACS_HLINE);
 
     wattrset(*win, DLGTEXTATTR);
-    mvwprintw(*win, 2, 2, "Select display field to sort by");
+    mvwprintw(*win, 2, 2, "Select sort criterion");
     wmove(*win, 4, 2);
-    printkeyhelp("P", " - total packets in", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("P", " - total packets in", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 5, 2);
-    printkeyhelp("I", " - IP packets in", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("I", " - IP packets in", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 6, 2);
-    printkeyhelp("B", " - total bytes in", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("B", " - total bytes in", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 7, 2);
-    printkeyhelp("K", " - total packets out", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("K", " - total packets out", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 8, 2);
-    printkeyhelp("O", " - IP packets out", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("O", " - IP packets out", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 9, 2);
-    printkeyhelp("Y", " - total bytes out", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("Y", " - total bytes out", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 10, 2);
-    printkeyhelp("Any other key", " - cancel sort", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("Any other key", " - cancel sort", *win, DLGHIGHATTR, DLGTEXTATTR);
     update_panels();
     doupdate();
 }
@@ -642,7 +669,7 @@ void sort_hosttab(struct ethtab *list, int *idx, int command)
     ptemp1 = list->firstvisible = list->head;
     *idx = 1;
     idxtmp = 0;
-    colorwin(list->tabwin);
+    tx_colorwin(list->tabwin);
     while ((ptemp1) && (idxtmp <= LINES - 4)) {
 	printethent(list, ptemp1, *idx);
 	idxtmp++;
@@ -743,8 +770,8 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
     hostmonhelp();
 
     initethtab(&table, options->actmode);
-    loaddesclist(&elist, LINK_ETHERNET);
-    loaddesclist(&flist, LINK_FDDI);
+    loaddesclist(&elist, LINK_ETHERNET, WITHETCETHERS);
+    loaddesclist(&flist, LINK_FDDI, WITHETCETHERS);
     
     if (logging) {
         if (strcmp(current_logfile, "") == 0)
@@ -795,7 +822,7 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
 		case 12:
 		case 'l':
 		case 'L':
-		    refresh_screen();
+		    tx_refresh_screen();
 		    break;
 		case 's':
 		case 'S':
@@ -827,9 +854,8 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
 	    }
 	    linktype = getlinktype(fromaddr.sll_hatype, ifname, -1, NULL);
 
-
 	    if ((linktype == LINK_ETHERNET) || (linktype == LINK_FDDI)
-		|| (linktype == LINK_PLIP)) {
+		|| (linktype == LINK_PLIP) || (linktype == LINK_TR)) {
 		if (fromaddr.sll_protocol == htons(ETH_P_IP))
 		    is_ip = 1;
 		else
@@ -851,7 +877,14 @@ void hostmon(const struct OPTIONS *options, int facilitytime, char *ifptr)
 		    memcpy(scratch_daddr, ((struct fddihdr *) buf)->daddr,
 			   FDDI_K_ALEN);
 		    list = &flist;
+		} else if (linktype == LINK_TR) {
+		    memcpy(scratch_saddr, ((struct trh_hdr *) buf)->saddr,
+		           TR_ALEN);
+		    memcpy(scratch_daddr, ((struct trh_hdr *) buf)->daddr,
+		           TR_ALEN);
+		    list = &flist;
 		}
+		
 		entry = in_ethtable(&table, linktype, scratch_saddr);
 
 		if ((entry == NULL) && (!nomem))
