@@ -6,24 +6,29 @@ struct ethtabent {
     int type;
     union {
 	struct {
-	    unsigned long inpcount;
-	    unsigned long inbcount;
-	    unsigned long inippcount;
+	    unsigned long long inpcount;
+	    unsigned long long inbcount;
+	    unsigned long long inippcount;
 	    unsigned long inspanbr;
 	    unsigned int inpktact;
-	    unsigned long outpcount;
-	    unsigned long outbcount;
-	    unsigned long outippcount;
+	    unsigned long long outpcount;
+	    unsigned long long outbcount;
+	    unsigned long long outippcount;
 	    unsigned long outspanbr;
 	    unsigned int outpktact;
 	    float inrate;
 	    float outrate;
+	    short past5;
 	} figs;
 
 	struct {
 	    char eth_addr[ETH_ALEN];
-	    char ascaddr[13];
+	    char ascaddr[15];
+	    char desc[65];
+	    char ifname[10];
+	    int withdesc;
 	    int printed;
+	    unsigned int linktype;
 	} desc;
     } un;
 
