@@ -3,7 +3,7 @@
 
 ifstats.c	- the interface statistics module
 Written by Gerard Paul Java
-Copyright (c) Gerard Paul Java 1997
+Copyright (c) Gerard Paul Java 1997-2002
 
 This software is open source; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@ details.
 #include <string.h>
 #include <signal.h>
 #include <unistd.h>
+#include <time.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -36,24 +37,30 @@ details.
 #include <netinet/tcp.h>
 #include <netinet/udp.h>
 #include <linux/if_ether.h>
+#include <winops.h>
+#include <labels.h>
+#include <listbox.h>
+#include <msgboxes.h>
 #include "ifstats.h"
 #include "ifaces.h"
 #include "isdntab.h"
+#include "fltdefs.h"
+#include "utfilter.h"
+#include "fltselect.h"
 #include "packet.h"
+#include "options.h"
 #include "log.h"
 #include "dirs.h"
-#include "stdwinset.h"
 #include "deskman.h"
 #include "ipcsum.h"
 #include "attrs.h"
-#include "error.h"
 #include "serv.h"
 #include "timer.h"
-#include "options.h"
 #include "instances.h"
 #include "mode.h"
 #include "logvars.h"
 #include "promisc.h"
+#include "error.h"
 
 #define SCROLLUP 0
 #define SCROLLDOWN 1
@@ -133,7 +140,7 @@ void initiflist(struct iflist **list)
 
     fd = open_procnetdev();
     if (fd == NULL) {
-	errbox("Unable to obtain interface list", ANYKEY_MSG, &resp);
+	tx_errbox("Unable to obtain interface list", ANYKEY_MSG, &resp);
 	return;
     }
 
@@ -232,6 +239,12 @@ void destroyiflist(struct iflist *list)
     }
 }
 
+void no_ifaces_error(void)
+{
+    write_error("No active interfaces.  Check their status or the /proc filesystem",
+                daemonized);
+}
+
 void updaterates(struct iftab *table, int unit, time_t starttime,
 		 time_t now, unsigned int idx)
 {
@@ -321,8 +334,6 @@ void labelstats(WINDOW * win)
 
 void initiftab(struct iftab *table)
 {
-    initiflist(&(table->head));
-
     table->borderwin = newwin(LINES - 2, COLS, 1, 0);
     table->borderpanel = new_panel(table->borderwin);
 
@@ -334,10 +345,10 @@ void initiftab(struct iftab *table)
     labelstats(table->borderwin);
     table->statwin = newwin(LINES - 4, COLS - 2, 2, 1);
     table->statpanel = new_panel(table->statwin);
-    stdwinset(table->statwin);
+    tx_stdwinset(table->statwin);
     wtimeout(table->statwin, -1);
     wattrset(table->statwin, STDATTR);
-    colorwin(table->statwin);
+    tx_colorwin(table->statwin);
     wattrset(table->statwin, BOXATTR);
     wmove(table->borderwin, LINES - 3, 32 * COLS / 80);
     wprintw(table->borderwin,
@@ -401,36 +412,31 @@ void pagegstatwin(struct iftab *table, int direction, int *idx)
  * The general interface statistics function
  */
 
-void ifstats(const struct OPTIONS *options, int facilitytime)
+void ifstats(const struct OPTIONS *options, struct filterstate *ofilter,
+    int facilitytime)
 {
     int logging = options->logging;
     struct iftab table;
 
-    struct isdntab isdntable;
-
     char buf[MAX_PACKET_SIZE];
-    char aligned_buf[120];
     char *packet;
-    struct iphdr *ipacket;
-    char *tpacket;
+    int pkt_result = 0;
 
     struct sockaddr_ll fromaddr;
     unsigned short linktype;
-
+    
+    unsigned sport, dport;
+    
     struct iflist *ptmp = NULL;
 
     unsigned int idx = 1;
 
     int fd;
     FILE *logfile = NULL;
-    int isdn_fd = -1;
 
     int br;
     char ifname[10];
 
-    int iphlen;
-    int ipck, ipcsum;
-
     int ch;
 
     struct timeval tv;
@@ -453,13 +459,20 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 	return;
     }
 
+    initiflist(&(table.head));
+    if (table.head == NULL) {
+        no_ifaces_error();
+        unmark_facility(GSTATIDFILE, "");
+        return;
+    }
+    
+    initiftab(&table);
     open_socket(&fd);
 
     if (fd < 0) {
 	unmark_facility(GSTATIDFILE, "");
 	return;
     }
-    initiftab(&table);
 
     if ((first_active_facility()) && (options->promisc)) {
 	init_promisc_list(&promisc_list);
@@ -498,8 +511,7 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 	update_panels();
 	doupdate();
 
-	bzero(&isdntable, sizeof(struct isdntab));
-
+        isdnfd = -1;
 	exitloop = 0;
 	gettimeofday(&tv, NULL);
 	starttime = startlog = statbegin = tv.tv_sec;
@@ -526,7 +538,7 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 		case 12:
 		case 'l':
 		case 'L':
-		    refresh_screen();
+		    tx_refresh_screen();
 		    break;
 
 		case 'Q':
@@ -540,6 +552,12 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 		}
 	    }
 	    if (br > 0) {
+                pkt_result = processpacket(buf, &packet, &br, NULL, &sport, &dport,
+                    &fromaddr, &linktype, ofilter, ifname, NULL);
+
+		if (pkt_result == INVALID_PACKET)
+		    continue;
+		
 		positionptr(&table, &ptmp, ifname);
 
 		ptmp->total++;
@@ -547,31 +565,13 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 		ptmp->spanbr += br;
 		ptmp->br += br;
 
-		if (fromaddr.sll_protocol == ntohs(ETH_P_IP)) {
-		    (ptmp->iptotal)++;
-
-		    isdn_iface_check(&isdn_fd, ifname);
-		    linktype = getlinktype(fromaddr.sll_hatype, ifname,
-					   isdn_fd, &isdntable);
-
-		    adjustpacket(buf, linktype, &packet, aligned_buf, &br);
-
-		    if (packet == NULL)
-			continue;
-
-		    ipacket = (struct iphdr *) packet;
-		    iphlen = ipacket->ihl * 4;
-		    tpacket = packet + iphlen;
-
-		    /* recalculate checksum */
-
-		    ipcsum = ipacket->check;
-		    ipacket->check = 0;
-		    ipck = in_cksum((u_short *) ipacket, iphlen);
-		    if ((ipcsum != ipck) || (!ipck)) {
+		if (fromaddr.sll_protocol == ETH_P_IP) {
+		    ptmp->iptotal++;
+		
+                    if (pkt_result == CHECKSUM_ERROR) {
 			(ptmp->badtotal)++;
 			continue;
-		    }
+	            }
 		} else {
 		    (ptmp->noniptotal)++;
 		}
@@ -635,7 +635,7 @@ void ifstats(const struct OPTIONS *options, int facilitytime)
 	fclose(logfile);
     }
     destroyiflist(table.head);
-    destroy_isdn_table(&isdntable);
+    pkt_cleanup();
     unmark_facility(GSTATIDFILE, "");
     strcpy(current_logfile, "");
 }
@@ -742,7 +742,8 @@ void printdetails(struct iftotals *totals, WINDOW * win)
  * The detailed interface statistics function
  */
 
-void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
+void detstats(char *iface, const struct OPTIONS *options, int facilitytime,
+              struct filterstate *ofilter)
 {
     int logging = options->logging;
 
@@ -750,32 +751,29 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
     PANEL *statpanel;
 
     char buf[MAX_PACKET_SIZE];
-    char aligned_buf[120];
     char *packet;
     struct iphdr *ipacket = NULL;
     char *tpacket;
+    unsigned int iphlen;
+    unsigned int sport, dport;
 
     char ifname[8];
     struct sockaddr_ll fromaddr;
     unsigned short linktype;
-
+    
     int fd;
-    int isdn_fd = -1;
     int br;
     int framelen = 0;
+    int pkt_result = 0;
 
     FILE *logfile = NULL;
     
-    int iphlen;
-    int ipck, ipcsum;
     unsigned int iplen = 0;
 
     struct iftotals totals;
 
     int ch;
 
-    struct isdntab isdntable;
-
     struct timeval tv;
     unsigned long updtime = 0;
     unsigned long long updtime_usec = 0;
@@ -807,9 +805,9 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
     char unitstring[7];
 
     struct promisc_states *promisc_list;
-    
     char err_msg[80];
 
+
     /*
      * Mark this facility
      */
@@ -853,10 +851,10 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
     stdexitkeyhelp();
     statwin = newwin(LINES - 2, COLS, 1, 0);
     statpanel = new_panel(statwin);
-    stdwinset(statwin);
+    tx_stdwinset(statwin);
     wtimeout(statwin, -1);
     wattrset(statwin, BOXATTR);
-    colorwin(statwin);
+    tx_colorwin(statwin);
     box(statwin, ACS_VLINE, ACS_HLINE);
     wmove(statwin, 0, 1);
     wprintw(statwin, " Statistics for %s ", iface);
@@ -894,13 +892,12 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 
     spanbr = 0;
 
-    bzero(&isdntable, sizeof(struct isdntab));
-
     gettimeofday(&tv, NULL);
     starttime = startlog = statbegin = tv.tv_sec;
 
     leaveok(statwin, TRUE);
 
+    isdnfd = -1;
     exitloop = 0;
     dispmode(options->actmode, unitstring);
 
@@ -916,7 +913,7 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 	    case 12:
 	    case 'l':
 	    case 'L':
-		refresh_screen();
+		tx_refresh_screen();
 		break;
 
 	    case 'Q':
@@ -930,54 +927,47 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
 	    }
 	}
 	if (br > 0) {
-	    if (strcmp(iface, ifname) != 0)
-		continue;
-
 	    framelen = br;
+            pkt_result = processpacket(buf, &packet, &br, NULL,
+                             &sport, &dport, &fromaddr, &linktype,
+                             ofilter, ifname, iface);
+            
+            if (pkt_result == INVALID_PACKET)
+                continue;
+                                 
 	    totals.total++;
-	    totals.bytestotal += br;
+	    totals.bytestotal += framelen;
 
 	    if (fromaddr.sll_pkttype == PACKET_OUTGOING) {
 		totals.total_out++;
-		totals.bytestotal_out += br;
-		spanbr_out += br;
+		totals.bytestotal_out += framelen;
+		spanbr_out += framelen;
 		spanpkt_out++;
 	    } else {
 		totals.total_in++;
-		totals.bytestotal_in += br;
-		spanbr_in += br;
+		totals.bytestotal_in += framelen;
+		spanbr_in += framelen;
 		spanpkt_in++;
 	    }
 
 	    if (fromaddr.sll_pkttype == PACKET_BROADCAST) {
 	        totals.bcast++;
-	        totals.bcastbytes += br;
+	        totals.bcastbytes += framelen;
 	    }
 	    
-	    spanbr += br;
+	    spanbr += framelen;
 	    spanpkt++;
 
-	    if (fromaddr.sll_protocol == ntohs(ETH_P_IP)) {
-		isdn_iface_check(&isdn_fd, ifname);
-		linktype = getlinktype(fromaddr.sll_hatype, ifname,
-				       isdn_fd, &isdntable);
+	    if (fromaddr.sll_protocol == ETH_P_IP) {
 
-		adjustpacket(buf, linktype, &packet, aligned_buf, &br);
-
-		if (packet == NULL)
-		    continue;
-
-		ipacket = (struct iphdr *) packet;
-		iphlen = ipacket->ihl * 4;
-		tpacket = packet + iphlen;
-
-		ipcsum = ipacket->check;
-		ipacket->check = 0;
-		ipck = in_cksum((u_short *) ipacket, iphlen);
-		if ((ipcsum != ipck) || (!ipck)) {
+		if (pkt_result == CHECKSUM_ERROR) {
 		    totals.badtotal++;
 		    continue;
 		}
+
+                ipacket = (struct iphdr *) packet;
+                iphlen = ipacket->ihl * 4;
+                tpacket = packet + iphlen;
 		iplen = ntohs(ipacket->tot_len);
 
 		totals.iptotal++;
@@ -1168,28 +1158,29 @@ void detstats(char *iface, const struct OPTIONS *options, int facilitytime)
     delwin(statwin);
     unmark_facility(DSTATIDFILE, iface);
     strcpy(current_logfile, "");
-    destroy_isdn_table(&isdntable);
+    pkt_cleanup();
     update_panels();
     doupdate();
 }
 
 void selectiface(char *ifname, int withall, int *aborted)
 {
-    WINDOW *win;
-    PANEL *panel;
-    WINDOW *bw;
-    PANEL *bp;
-
-    int row;
     int ch;
 
-    int endloop = 0;
-
     struct iflist *list;
     struct iflist *ptmp;
 
+    struct scroll_list scrolllist;
+    
     initiflist(&list);
-    if (withall) {
+    
+    if (list == NULL) {
+        no_ifaces_error();
+        *aborted = 1;
+        return;
+    }
+        
+    if ((withall) && (list != NULL)) {
 	ptmp = malloc(sizeof(struct iflist));
 	strcpy(ptmp->ifname, "All interfaces");
 
@@ -1198,111 +1189,34 @@ void selectiface(char *ifname, int withall, int *aborted)
 	ptmp->next_entry = list;
 	list = ptmp;
     }
-    listkeyhelp();
-    bw = newwin(14, 24, (LINES - 14) / 2, (COLS - 24) / 2 - 9);
-    /* I'm guessing (COLS-24)/2-9*COLS/80) is overkill */
-    bp = new_panel(bw);
-    wattrset(bw, BOXATTR);
-    box(bw, ACS_VLINE, ACS_HLINE);
+    tx_listkeyhelp(STDATTR, HIGHATTR);
 
-    win = newwin(12, 22, (LINES - 14) / 2 + 1, (COLS - 24) / 2 - 8);
-    panel = new_panel(win);
-    stdwinset(win);
-    wattrset(win, BOXATTR);
-    colorwin(win);
-    wmove(bw, 0, 1);
-    wprintw(bw, " Select Interface ");
-
-    row = 0;
     ptmp = list;
 
-    wattrset(win, STDATTR);
-    do {
-	wmove(win, row, 2);
-	wprintw(win, "%s", ptmp->ifname);
-	ptmp = ptmp->next_entry;
-	row++;
-    } while ((ptmp != NULL) && (row <= 11));
-    update_panels();
-    doupdate();
-
-    row = 0;
-    ptmp = list;
-
-    do {
-	wattrset(win, PTRATTR);
-	wmove(win, row, 1);
-	waddch(win, ACS_RARROW);
-	ch = wgetch(win);
-	wmove(win, row, 1);
-	wprintw(win, " ");
-
-	wattrset(win, STDATTR);
-	switch (ch) {
-	case KEY_DOWN:
-	    if (ptmp->next_entry != NULL) {
-		if (row < 11)
-		    row++;
-		else {
-		    wscrl(win, 1);
-		    scrollok(win, 0);
-		    wmove(win, 11, 0);
-		    wprintw(win, "%22c", ' ');
-		    wmove(win, 11, 2);
-		    wprintw(win, ptmp->next_entry->ifname);
-		    scrollok(win, 1);
-		}
-		ptmp = ptmp->next_entry;
-	    }
-	    break;
-	case KEY_UP:
-	    if (ptmp->prev_entry != NULL) {
-		if (row > 0)
-		    row--;
-		else {
-		    wscrl(win, -1);
-		    wmove(win, 0, 0);
-		    wprintw(win, "%22c", ' ');
-		    wmove(win, 0, 2);
-		    wprintw(win, ptmp->prev_entry->ifname);
-		}
-		ptmp = ptmp->prev_entry;
-	    }
-	    break;
-	case 13:
-	    endloop = 1;
-	    *aborted = 0;
-	    break;
-	case 12:
-	case 'l':
-	case 'L':
-	    refresh_screen();
-	    break;
-	case 27:
-	case 24:
-	case 'X':
-	case 'x':
-	case 'Q':
-	case 'q':
-	    endloop = 1;
-	    *aborted = 1;
-	    break;
-	}
-	update_panels();
-	doupdate();
-    } while (!endloop);
-
-    if (!(*aborted)) {
+    tx_init_listbox(&scrolllist, 24, 14, (COLS - 24) / 2 - 9, (LINES - 14) / 2,
+        STDATTR, BOXATTR, BARSTDATTR, HIGHATTR);
+    
+    tx_set_listbox_title(&scrolllist, "Select Interface", 1);
+    
+    while (ptmp != NULL) {
+        tx_add_list_entry(&scrolllist, (char *) ptmp, ptmp->ifname);
+        ptmp = ptmp->next_entry;
+    }
+    
+    tx_show_listbox(&scrolllist);
+    tx_operate_listbox(&scrolllist, &ch, aborted);
+    tx_close_listbox(&scrolllist);
+    
+    if (!(*aborted) && (list != NULL)) {
+        ptmp = (struct iflist *) scrolllist.textptr->nodeptr;
 	if ((withall) && (ptmp->prev_entry == NULL))	/* All Interfaces */
 	    strcpy(ifname, "");
 	else
 	    strcpy(ifname, ptmp->ifname);
     }
+    
+    tx_destroy_list(&scrolllist);
     destroyiflist(list);
-    del_panel(panel);
-    delwin(win);
-    del_panel(bp);
-    delwin(bw);
     update_panels();
     doupdate();
 }
