@@ -30,7 +30,7 @@ if [ ! -d $WORKDIR ]; then
 else
     echo ">>> IPTraf work directory $WORKDIR already exists"
     echo "*** Clearing non-TCP filter data file"
-    rm -f $WORKDIR/othfilters.dat
+    rm -f $WORKDIR/othfilter.dat
 fi
 $INSTALL -m 0700 -o root -g root -d $WORKDIR
 
@@ -61,7 +61,7 @@ echo
 echo -n "Would you like to view the RELEASE-NOTES file now (Y/N)? "; read YESNO
 
 if [ "$YESNO" = "y" -o "$YESNO" = "Y" ]; then
-    less ../RELEASE-NOTES
+	less ../RELEASE-NOTES
 fi
 
 clear
