@@ -21,16 +21,18 @@ details.
 #include <errno.h>
 #include <signal.h>
 #include <ctype.h>
+#include <winops.h>
+#include <labels.h>
+#include "options.h"
 #include "tcptable.h"
 #include "othptab.h"
-#include "isdntab.h"
-#include "packet.h"
-#include "ifaces.h"
-#include "ipcsum.h"
-#include "promisc.h"
 #include "fltdefs.h"
 #include "utfilter.h"
 #include "fltselect.h"
+#include "isdntab.h"
+#include "packet.h"
+#include "ifaces.h"
+#include "promisc.h"
 #include "deskman.h"
 #include "error.h"
 #include "attrs.h"
@@ -40,9 +42,9 @@ details.
 #include "dirs.h"
 #include "timer.h"
 #include "ipfrag.h"
-#include "options.h"
 #include "instances.h"
 #include "logvars.h"
+#include "bar.h"
 
 #define SCROLLUP 0
 #define SCROLLDOWN 1
@@ -53,7 +55,6 @@ extern int daemonized;
 void writetcplog(int logging, FILE * fd, struct tcptableent *entry,
 		 unsigned int pktlen, int mac, char *message);
 void write_tcp_unclosed(int logging, FILE * fd, struct tcptable *table);
-void write_timeout_log(int logging, FILE *logfile, struct tcptableent *tcpnode);
 
 void rotate_ipmon_log()
 {
@@ -67,10 +68,10 @@ void rotate_ipmon_log()
 void ipmonhelp()
 {
     move(LINES - 1, 1);
-    printkeyhelp("Up/Dn/PgUp/PgDn", "-scroll  ", stdscr, HIGHATTR, STATUSBARATTR);
+    tx_printkeyhelp("Up/Dn/PgUp/PgDn", "-scroll  ", stdscr, HIGHATTR, STATUSBARATTR);
     move(LINES - 1, 43);
-    printkeyhelp("W", "-chg actv win  ", stdscr, HIGHATTR, STATUSBARATTR);
-    printkeyhelp("S", "-sort TCP  ", stdscr, HIGHATTR, STATUSBARATTR);
+    tx_printkeyhelp("W", "-chg actv win  ", stdscr, HIGHATTR, STATUSBARATTR);
+    tx_printkeyhelp("S", "-sort TCP  ", stdscr, HIGHATTR, STATUSBARATTR);
     stdexitkeyhelp();
 };
 
@@ -78,26 +79,19 @@ void uniq_help(int what)
 {
     move(LINES - 1, 25);
     if (!what)
-	printkeyhelp("M", "-more TCP info   ", stdscr, HIGHATTR, STATUSBARATTR);
+	tx_printkeyhelp("M", "-more TCP info   ", stdscr, HIGHATTR, STATUSBARATTR);
     else
-	printkeyhelp("Lft/Rt", "-vtcl scrl  ", stdscr, HIGHATTR, STATUSBARATTR);
+	tx_printkeyhelp("Lft/Rt", "-vtcl scrl  ", stdscr, HIGHATTR, STATUSBARATTR);
 }
 
 /* Mark general packet count indicators */
 
-void prepare_ip_statwin(WINDOW * win)
+void prepare_statwin(WINDOW * win)
 {
     wattrset(win, IPSTATLABELATTR);
     wmove(win, 0, 1);
-    wprintw(win, "IP:");
-    wmove(win, 0, 16 * COLS / 80);
-    wprintw(win, "TCP:");
-    wmove(win, 0, 31 * COLS / 80);
-    wprintw(win, "UDP:");
-    wmove(win, 0, 46 * COLS / 80);
-    wprintw(win, "ICMP:");
-    wmove(win, 0, 62 * COLS / 80);
-    wprintw(win, "Non-IP:");
+    wprintw(win, "Pkts captured (all interfaces):");
+    mvwaddch(win, 0, 45 * COLS / 80, ACS_VLINE);
 }
 
 void markactive(int curwin, WINDOW * tw, WINDOW * ow)
@@ -125,23 +119,11 @@ void markactive(int curwin, WINDOW * tw, WINDOW * ow)
     whline(win2, ACS_HLINE, 8);
 }
 
-void show_ip_stats(WINDOW * win, unsigned long long iptotal,
-		   unsigned long long tcptotal,
-		   unsigned long long udptotal,
-		   unsigned long long icmptotal,
-		   unsigned long long noniptotal)
+void show_stats(WINDOW * win, unsigned long long total)
 {
     wattrset(win, IPSTATATTR);
-    wmove(win, 0, 6 * COLS / 80);
-    printlargenum(iptotal, win);
-    wmove(win, 0, 21 * COLS / 80);
-    printlargenum(tcptotal, win);
-    wmove(win, 0, 36 * COLS / 80);
-    printlargenum(udptotal, win);
-    wmove(win, 0, 52 * COLS / 80);
-    printlargenum(icmptotal, win);
-    wmove(win, 0, 70 * COLS / 80);
-    printlargenum(noniptotal, win);
+    wmove(win, 0, 35 * COLS / 80);
+    printlargenum(total, win);
 }
 
 
@@ -190,13 +172,17 @@ void pageupperwin(struct tcptable *table, int direction,
 	while ((i <= table->imaxy - 3)
 	       && (table->lastvisible != table->tail)) {
 	    i++;
-	    scrollupperwin(table, direction, idx, mode);
+	    table->firstvisible = table->firstvisible->next_entry;
+	    table->lastvisible = table->lastvisible->next_entry;
+	    (*idx)++;
 	}
     } else {
 	while ((i <= table->imaxy - 3)
 	       && (table->firstvisible != table->head)) {
 	    i++;
-	    scrollupperwin(table, direction, idx, mode);
+	    table->firstvisible = table->firstvisible->prev_entry;
+	    table->lastvisible = table->lastvisible->prev_entry;
+	    (*idx)--;
 	}
     }
 }
@@ -246,13 +232,27 @@ void pagelowerwin(struct othptable *table, int direction)
 	while ((i <= table->oimaxy - 2)
 	       && (table->lastvisible != table->tail)) {
 	    i++;
-	    scrolllowerwin(table, direction);
+	    table->firstvisible = table->firstvisible->next_entry;
+	    table->lastvisible = table->lastvisible->next_entry;
+
+	    if (table->htstat == HIND) {	/* Head indicator on? */
+		wmove(table->borderwin, table->obmaxy - 1, 1);
+		whline(table->borderwin, ACS_HLINE, 8);
+		table->htstat = NOHTIND;
+	    }
 	}
     } else {
 	while ((i <= table->oimaxy - 2)
 	       && (table->firstvisible != table->head)) {
 	    i++;
-	    scrolllowerwin(table, direction);
+	    table->firstvisible = table->firstvisible->prev_entry;
+	    table->lastvisible = table->lastvisible->prev_entry;
+
+	    if (table->htstat == TIND) {	/* Tail indicator on? */
+		wmove(table->borderwin, table->obmaxy - 1, 1);
+		whline(table->borderwin, ACS_HLINE, 8);
+		table->htstat = NOHTIND;
+	    }
 	}
     }
 }
@@ -267,16 +267,16 @@ void show_tcpsort_win(WINDOW ** win, PANEL ** panel)
     *panel = new_panel(*win);
 
     wattrset(*win, DLGBOXATTR);
-    colorwin(*win);
+    tx_colorwin(*win);
     box(*win, ACS_VLINE, ACS_HLINE);
     wattrset(*win, DLGTEXTATTR);
     mvwprintw(*win, 2, 2, "Select sort criterion");
     wmove(*win, 4, 2);
-    printkeyhelp("P", " - sort by packet count", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("P", " - sort by packet count", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 5, 2);
-    printkeyhelp("B", " - sort by byte count", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("B", " - sort by byte count", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 6, 2);
-    printkeyhelp("Any other key", " - cancel sort", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("Any other key", " - cancel sort", *win, DLGHIGHATTR, DLGTEXTATTR);
     update_panels();
     doupdate();
 }
@@ -340,7 +340,8 @@ unsigned long long qt_getkey(struct tcptableent *entry, int ch)
 struct tcptableent *qt_partition(struct tcptable *table,
                                  struct tcptableent **low,
                                  struct tcptableent **high,
-                                 int ch, time_t timeout, int logging,
+                                 int ch, struct OPTIONS *opts,
+                                 int logging,
                                  FILE *logfile, int *nomem)
 {
     struct tcptableent *pivot = *low;
@@ -364,13 +365,13 @@ struct tcptableent *qt_partition(struct tcptable *table,
             /*
              * Might as well check out timed out entries here too.
              */   
-            if ((timeout > 0) && ((now - left->lastupdate) / 60 > timeout) &&
+            if ((opts->timeout > 0) && ((now - left->lastupdate) / 60 > opts->timeout) &&
                (!(left->inclosed))) {
                 left->timedout = left->oth_connection->timedout = 1;
                 addtoclosedlist(table, left, nomem);
                 
                 if (logging)
-                    write_timeout_log(logging, logfile, left);
+                    write_timeout_log(logging, logfile, left, opts);
             }
             
             left = left->next_entry->next_entry;
@@ -380,13 +381,13 @@ struct tcptableent *qt_partition(struct tcptable *table,
             /*
              * Might as well check out timed out entries here too.
              */   
-            if ((timeout > 0) && ((now - right->lastupdate) / 60 > timeout) &&
+            if ((opts->timeout > 0) && ((now - right->lastupdate) / 60 > opts->timeout) &&
                (!(right->inclosed))) {
                 right->timedout = right->oth_connection->timedout = 1;
                 addtoclosedlist(table, right, nomem);
                 
                 if (logging)
-                    write_timeout_log(logging, logfile, right);
+                    write_timeout_log(logging, logfile, right, opts);
             }
             right = right->prev_entry->prev_entry;
         }
@@ -420,8 +421,10 @@ struct tcptableent *qt_partition(struct tcptable *table,
  * Quicksort the TCP entries.
  */
 void quicksort_tcp_entries(struct tcptable *table, struct tcptableent *low,
-                           struct tcptableent *high, int ch, time_t timeout,
-                           int logging, FILE *logfile, int *nomem)
+                           struct tcptableent *high, int ch,
+                           struct OPTIONS *opts,
+                           int logging,
+                           FILE *logfile, int *nomem)
 {
     struct tcptableent *pivot;
     
@@ -429,15 +432,15 @@ void quicksort_tcp_entries(struct tcptable *table, struct tcptableent *low,
         return;
         
     if (high->index > low->index) {
-        pivot = qt_partition(table, &low, &high, ch, timeout,
+        pivot = qt_partition(table, &low, &high, ch, opts,
                              logging, logfile, nomem);
         
         if (pivot->prev_entry != NULL)
             quicksort_tcp_entries(table, low, pivot->prev_entry->prev_entry,
-                                  ch, timeout, logging, logfile, nomem);
+                                  ch, opts, logging, logfile, nomem);
         
         quicksort_tcp_entries(table, pivot->next_entry->next_entry, high,
-                              ch, timeout, logging, logfile, nomem);
+                              ch, opts, logging, logfile, nomem);
     }
 }
 
@@ -448,7 +451,7 @@ void quicksort_tcp_entries(struct tcptable *table, struct tcptableent *low,
  
 void sortipents(struct tcptable *table, unsigned long *idx, int ch,
 		int mode, int logging, FILE *logfile,
-		time_t timeout, int *nomem)
+		time_t timeout, int *nomem, struct OPTIONS *opts)
 {
     struct tcptableent *tcptmp1;
     unsigned int idxtmp;
@@ -462,21 +465,22 @@ void sortipents(struct tcptable *table, unsigned long *idx, int ch,
         return;
             
     quicksort_tcp_entries(table, table->head, table->tail->prev_entry,
-                          ch, timeout, logging, logfile, nomem);
+                          ch, opts, logging, logfile, nomem);
      
     update_panels();
     doupdate();
-    colorwin(table->tcpscreen);
+    tx_colorwin(table->tcpscreen);
 
     tcptmp1 = table->firstvisible = table->head;
     *idx = 1;
     idxtmp = 0;
+
     while ((tcptmp1 != NULL) && (idxtmp <= table->imaxy - 1)) {
-	printentry(table, tcptmp1, *idx, mode);
 	if (idxtmp++ <= table->imaxy - 1)
 	    table->lastvisible = tcptmp1;
 	tcptmp1 = tcptmp1->next_entry;
     }
+
 }
 
 /*
@@ -526,30 +530,47 @@ int checkrvnamed(void)
     return 1;
 }
 
+void update_flowrate(WINDOW *win, struct tcptableent *entry, time_t now,
+    int *cleared, int mode)
+{
+    float rate = 0;
+    char units[10];
+    
+    wattrset(win, IPSTATLABELATTR);
+    mvwprintw(win, 0, COLS * 47 / 80, "TCP flow rate: ");
+    wattrset(win, IPSTATATTR);
+    if (mode == KBITS) {
+        strcpy(units, "kbits/s");
+        rate = (float) (entry->spanbr * 8 / 1000) / (float) (now - entry->starttime);
+    } else {
+        strcpy(units, "kbytes/s");
+        rate = (float) (entry->spanbr / 1024) / (float) (now - entry->starttime);
+    }
+    mvwprintw(win, 0, COLS * 50 / 80 + 13, "%8.2f %s", rate, units);
+    entry->spanbr = 0;
+    *cleared = 0;
+}
+     
 /* 
  * The IP Traffic Monitor
  */
 
-void ipmon(const struct OPTIONS *options,
-	   struct filterstate *ofilter,
-	   unsigned int tcptm, int facilitytime, char *ifptr)
+void ipmon(struct OPTIONS *options,
+	   struct filterstate *ofilter, int facilitytime, char *ifptr)
 {
     int logging = options->logging;
     struct sockaddr_ll fromaddr;	/* iface info */
     unsigned short linktype;	/* data link type */
     int fd;			/* raw socket */
-    int isdn_fd = -1;		/* file descriptor for isdnctrl if any */
     char tpacket[MAX_PACKET_SIZE];	/* raw packet data */
-    char aligned_buf[ALIGNED_BUF_LEN];	/* aligned buffer */
     char *packet = NULL;	/* network packet ptr */
     struct iphdr *ippacket;
     struct tcphdr *transpacket;	/* IP-encapsulated packet */
     unsigned int sport = 0, dport = 0;	/* TCP/UDP port values */
-    int firstin = 0;
     char sp_buf[10];
 
     unsigned long screen_idx = 1;
-
+    
     struct timeval tv;
     unsigned long starttime = 0;
     unsigned long now = 0;
@@ -572,34 +593,24 @@ void ipmon(const struct OPTIONS *options,
     int readlen;
     char ifname[8];
 
-    unsigned long long iptotal = 0;
-    unsigned long long noniptotal = 0;
-    unsigned long long tcptotal = 0;
-    unsigned long long udptotal = 0;
-    unsigned long long icmptotal = 0;
-    unsigned long long othptotal = 0;
-
-    unsigned int nbplen;
-    unsigned int br;
-
-    int hdrchk;			/* IP checksum values */
-    int hdrcsum;
-
+    unsigned long long total_pkts = 0;
+    
+    unsigned int br;    	/* bytes read.  Differs from readlen */
+                        	/* only when packets fragmented */
     unsigned int iphlen;
     unsigned int totalhlen;
 
     struct tcptable table;
     struct tcptableent *tcpentry;
+    struct tcptableent *tmptcp;
     int mode = 0;
 
     struct othptable othptbl;
     struct othptabent *othpent;
 
-    struct isdntab isdntable;
-
     int p_sstat = 0, p_dstat = 0;	/* Reverse lookup statuses prior to */
-    					/* reattempt in updateentry(). */
-    int nifltok = 0;		/* Non-IP filter ok */
+    					/* reattempt in updateentry() */
+    int pkt_result = 0;		/* Non-IP filter ok */
 
     int fragment = 0;		/* Set to 1 if not first fragment */
 
@@ -614,6 +625,10 @@ void ipmon(const struct OPTIONS *options,
 
     int instance_id;
     int revlook = options->revlook;
+    int statcleared = 0;
+    int wasempty = 1;
+    
+    const int statx = COLS * 47 / 80;
     
     /* 
      * Mark this instance of the traffic monitor
@@ -666,31 +681,33 @@ void ipmon(const struct OPTIONS *options,
     init_tcp_table(&table);
     init_othp_table(&othptbl, options->mac);
 
-    if (revlook) {
-	if (checkrvnamed())
-	    open_rvn_socket(&rvnfd);
-    } else
-	rvnfd = 0;
-
-    if (options->servnames)
-	setservent(1);
-
-    ipmonhelp();
-    uniq_help(0);
     statwin = newwin(1, COLS, LINES - 2, 0);
     statpanel = new_panel(statwin);
     wattrset(statwin, IPSTATLABELATTR);
     wmove(statwin, 0, 0);
     sprintf(sp_buf, "%%%dc", COLS);
     wprintw(statwin, sp_buf, ' ');
-    prepare_ip_statwin(statwin);
-    show_ip_stats(statwin, 0, 0, 0, 0, 0);
+    prepare_statwin(statwin);
+    show_stats(statwin, 0);
     markactive(curwin, table.borderwin, othptbl.borderwin);
     update_panels();
     doupdate();
 
-    bzero(&isdntable, sizeof(struct isdntab));
-    
+    if (revlook) {
+	if (checkrvnamed())
+	    open_rvn_socket(&rvnfd);
+    } else
+	rvnfd = 0;
+
+    ipmonhelp();
+    uniq_help(0);
+
+    update_panels();
+    doupdate();
+
+    if (options->servnames)
+	setservent(1);
+
     /*
      * Try to open log file if logging activated.  Turn off logging
      * (for this session only) if an error was discovered in opening
@@ -718,11 +735,86 @@ void ipmon(const struct OPTIONS *options,
     rotate_flag = 0;
     writelog(logging, logfile, "******** IP traffic monitor started ********");
    
+    isdnfd = -1;
     exitloop = 0;
     gettimeofday(&tv, NULL);
     starttime = timeint = closedint = tv.tv_sec;
 
     while (!exitloop) {
+	gettimeofday(&tv, NULL);
+	now = tv.tv_sec;
+	unow = tv.tv_sec * 1e+06 + tv.tv_usec;
+        
+        /* 
+         * Print timer at bottom of screen
+         */
+         
+	if (now - timeint >= 5) {
+	    printelapsedtime(starttime, now, othptbl.obmaxy - 1, 15,
+			     othptbl.borderwin);
+	    timeint = now;
+	}
+
+	/*
+	 * Automatically clear closed/timed out entries
+	 */
+
+	if ((options->closedint != 0) &&
+	    ((now - closedint) / 60 >= options->closedint)) {
+	    flushclosedentries(&table, &screen_idx,
+			       logging, logfile, options);
+	    refreshtcpwin(&table, screen_idx, mode);
+	    closedint = now;
+	}
+
+	/*
+	 * Update screen at configured intervals.
+	 */
+
+	if (((options->updrate != 0)
+	     && (now - updtime >= options->updrate))
+	    || ((options->updrate == 0)
+		&& (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
+	    update_panels();
+	    doupdate();
+	    updtime = now;
+	    updtime_usec = unow;
+	}
+	
+	/*
+	 * If highlight bar is on some entry, update the flow rate
+	 * indicator after five seconds.
+	 */
+	if (table.barptr != NULL) {
+	    if ((now - table.barptr->starttime) >= 5) {
+	        update_flowrate(statwin, table.barptr, now, &statcleared,
+	            options->actmode);
+	        table.barptr->starttime = now;
+	    }
+	} else {
+	    wattrset(statwin, IPSTATATTR);
+	    mvwprintw(statwin, 0, statx, "No TCP entries              ");
+	}
+	
+	/*
+	 * Terminate facility should a lifetime be specified at the
+	 * command line
+	 */
+	if ((facilitytime != 0)
+	    && (((now - starttime) / 60) >= facilitytime))
+	    exitloop = 1;
+
+        /*
+         * Close and rotate log file if signal was received
+         */
+	if ((rotate_flag == 1) && (logging)) {
+	    announce_rotate_prepare(logfile);
+	    write_tcp_unclosed(logging, logfile, &table);
+	    rotate_logfile(&logfile, target_logname);
+	    announce_rotate_complete(logfile);
+	    rotate_flag = 0;
+	}
+
 	getpacket(fd, tpacket, &fromaddr, &ch, &readlen, ifname,
 		  table.tcpscreen);
 
@@ -730,17 +822,52 @@ void ipmon(const struct OPTIONS *options,
 	    if (keymode == 0) {
 		switch (ch) {
 		case KEY_UP:
-		    if (!curwin)
-			scrollupperwin(&table, SCROLLDOWN, &screen_idx,
-				       mode);
-		    else
+		    if (!curwin) {
+		        if (table.barptr != NULL) {
+		            if (table.barptr->prev_entry != NULL) {
+		                tmptcp = table.barptr;
+		                set_barptr((char **) &(table.barptr),
+		                    (char *) table.barptr->prev_entry,
+		                    &(table.barptr->prev_entry->starttime),
+		                    (char *) &(table.barptr->prev_entry->spanbr),
+		                    sizeof(unsigned long), statwin, &statcleared, statx);
+		                
+		                printentry(&table, tmptcp, screen_idx,  mode);
+
+		                if (table.baridx == 1)
+			            scrollupperwin(&table, SCROLLDOWN, &screen_idx,
+		  		           mode);
+			        else
+			            (table.baridx)--;
+
+			        printentry(&table, table.barptr, screen_idx, mode);
+		            }
+		        }
+		    } else
 			scrolllowerwin(&othptbl, SCROLLDOWN);
 		    break;
 		case KEY_DOWN:
-		    if (!curwin)
-			scrollupperwin(&table, SCROLLUP, &screen_idx,
-				       mode);
-		    else
+		    if (!curwin) {
+		        if (table.barptr != NULL) {
+		            if (table.barptr->next_entry != NULL) {
+		                tmptcp = table.barptr;
+		                set_barptr((char **) &(table.barptr), 
+		                    (char *) table.barptr->next_entry,
+		                    &(table.barptr->next_entry->starttime),
+		                    (char *) &(table.barptr->next_entry->spanbr),
+		                    sizeof(unsigned long), statwin, &statcleared, statx);
+		                printentry(&table, tmptcp, screen_idx, mode);
+		            
+		                if (table.baridx == table.imaxy)
+			            scrollupperwin(&table, SCROLLUP, &screen_idx,
+				           mode);
+			        else
+			            (table.baridx)++;
+			        
+			        printentry(&table, table.barptr, screen_idx, mode);
+			    }
+			}
+		    } else
 			scrolllowerwin(&othptbl, SCROLLUP);
 		    break;
 		case KEY_RIGHT:
@@ -761,31 +888,53 @@ void ipmon(const struct OPTIONS *options,
 		    break;
 		case KEY_PPAGE:
 		case '-':
-		    if (!curwin)
-			pageupperwin(&table, SCROLLDOWN, &screen_idx,
-				     mode);
-		    else
+		    if (!curwin) {
+		        if (table.barptr != NULL) {
+			    pageupperwin(&table, SCROLLDOWN, &screen_idx,
+			  	         mode);
+                            set_barptr((char **) &(table.barptr),
+                                (char *) table.lastvisible,
+                                &(table.lastvisible->starttime),
+                                (char *) &(table.lastvisible->spanbr),
+                                sizeof(unsigned long), statwin, &statcleared, statx);
+                            table.baridx = table.lastvisible->index - screen_idx + 1;
+                            refreshtcpwin(&table, screen_idx, mode);
+                        }
+		    } else {
 			pagelowerwin(&othptbl, SCROLLDOWN);
+			refresh_othwindow(&othptbl);
+	            }
 		    break;
 		case KEY_NPAGE:
 		case ' ':
-		    if (!curwin)
-			pageupperwin(&table, SCROLLUP, &screen_idx, mode);
-		    else
+		    if (!curwin) {
+		        if (table.barptr != NULL) {
+			    pageupperwin(&table, SCROLLUP, &screen_idx, mode);
+                            set_barptr((char **) &(table.barptr), 
+                                (char *) table.firstvisible,
+                                &(table.firstvisible->starttime),
+                                (char *) &(table.firstvisible->spanbr),
+                                sizeof(unsigned long), statwin, &statcleared, statx);
+                            table.baridx =  1;
+                            refreshtcpwin(&table, screen_idx, mode);
+                        }
+		    } else {
 			pagelowerwin(&othptbl, SCROLLUP);
+			refresh_othwindow(&othptbl);
+		    }
 		    break;
 		case KEY_F(6):
 		case 'w':
 		case 'W':
 		case 9:
-		    curwin = ~curwin;
+		    curwin = !curwin;
 		    markactive(curwin, table.borderwin, othptbl.borderwin);
 		    uniq_help(curwin);
 		    break;
 		case 'm':
 		case 'M':
 		    if (!curwin) {
-			mode = (++mode) % 3;
+			mode = (mode + 1) % 3;
 			if ((mode == 1) && (!options->mac))
 			    mode = 2;
 			refreshtcpwin(&table, screen_idx, mode);
@@ -794,15 +943,15 @@ void ipmon(const struct OPTIONS *options,
 		case 12:
 		case 'l':
 		case 'L':
-		    refresh_screen();
+		    tx_refresh_screen();
 		    break;
 
 		case 'F':
 		case 'f':
 		case 'c':
 		case 'C':
-		    flushclosedentries(&table, tcptm, &screen_idx,
-				       logging, logfile);
+		    flushclosedentries(&table, &screen_idx,
+				       logging, logfile, options);
 		    refreshtcpwin(&table, screen_idx, mode);
 		    break;
 		case 's':
@@ -827,130 +976,54 @@ void ipmon(const struct OPTIONS *options,
 		update_panels();
 		doupdate();
 		sortipents(&table, &screen_idx, ch, mode, logging, logfile,
-		           options->timeout, &nomem);
+		           options->timeout, &nomem, options);
+		
+		if (table.barptr != NULL) {
+		    set_barptr((char **) &(table.barptr), (char *) table.firstvisible,
+		        &(table.firstvisible->starttime),
+		        (char *) &(table.firstvisible->spanbr), sizeof(unsigned long),
+		        statwin, &statcleared, statx);
+		    table.baridx = 1;
+		}
+		refreshtcpwin(&table, screen_idx, mode);
 		del_panel(sortpanel);
 		delwin(sortwin);
 		update_panels();
 		doupdate();
             }
 	}
+	
 	if (readlen > 0) {
-	    if (!iface_supported(ifname))
-		continue;
+            total_pkts++;
+  	    show_stats(statwin, total_pkts);
+  	    
+            pkt_result = processpacket((char *) tpacket, &packet, &readlen,
+                                       &br, &sport, &dport, &fromaddr,
+                                       &linktype, ofilter, ifname, ifptr);            
 
-	    /* Is is for any particular interface only? */
-
-	    if (ifptr != NULL) {
-		if (strcmp(ifname, ifptr) != 0)
-		    continue;
-	    }
-	    isdn_iface_check(&isdn_fd, ifname);
-	    linktype = getlinktype(fromaddr.sll_hatype, ifname,
-				   isdn_fd, &isdntable);
-
-	    fromaddr.sll_protocol = htons(fromaddr.sll_protocol);
-	    adjustpacket(tpacket, linktype, &packet, aligned_buf,
-			 &readlen);
-
-	    if (packet == NULL)
-		continue;
-
-	    if (fromaddr.sll_protocol != ETH_P_IP) {
-		if ((fromaddr.sll_protocol == ETH_P_ARP) ||
-		    (fromaddr.sll_protocol == ETH_P_RARP))
-		    nifltok = othfilterok(ofilter, fromaddr.sll_protocol,
-					  0, 0, 0, 0);
-		else
-		    nifltok = othfilterok(ofilter, 0, 0, 0, 0, 0);
-
-
-		/*
-		 * Display non-IP entry in lower window if filter ok,
-		 * and nomem not set
-		 */
-
-
-		if ((nifltok) && (!nomem))
-		    othpent = add_othp_entry(&othptbl, &table,
+            if (pkt_result != PACKET_OK)
+                continue;
+                
+            if (fromaddr.sll_protocol != ETH_P_IP) {
+		othpent = add_othp_entry(&othptbl, &table,
 					     0, 0, NOT_IP,
 					     fromaddr.sll_protocol,
 					     linktype, (char *) tpacket,
-					     (char *) packet, readlen, ifname, 0, 0,
+					     (char *) packet, br, ifname, 0, 0,
 					     0, logging, logfile,
 					     options->servnames, 0,
 					     &nomem);
+	        continue;
+	    } else {
+	        ippacket = (struct iphdr *) packet;
+	        iphlen = ippacket->ihl * 4;
+	        transpacket = (struct tcphdr *) (packet + iphlen);
 
-		noniptotal += readlen;
-		continue;
-	    }
-	    /* 
-	     * move past data link header, if present
-	     */
+	        if (ippacket->protocol == IPPROTO_TCP) {
 
-	    ippacket = (struct iphdr *) packet;
-	    iphlen = ippacket->ihl * 4;
-	    transpacket = (struct tcphdr *) (packet + iphlen);
-
-	    /*
-	     * Compute and verify IP header checksum
-	     */
-
-	    hdrcsum = ippacket->check;
-	    ippacket->check = 0;
-	    hdrchk = in_cksum((u_short *) ippacket, iphlen);
-
-	    if (hdrcsum != hdrchk)
-		continue;
-
-	    nbplen = ntohs(ippacket->tot_len);
-
-	    iptotal += nbplen;
-	    if (ippacket->protocol == IPPROTO_TCP) {
-		tcptotal += ntohs(ippacket->tot_len);
-
-		/*
-		 * TCP fragment handling policy:
-		 *    IP fragment offset == 0 && !(more frags); continue
-		 *    IP fragment offset != 0 {
-		 *        pass fragment to fragment processor
-		 *        if first fragment already in {
-		 *            pass sport, dport, and bytes
-		 *        }
-		 *        if first frag not in; get next frame
-		 *    }
-		 *       DM
-		 *      011 0 0000 0000 0000 
-		 *      001 1 1111 1111 1111
-		 *          3    F    F    F
-		 */
-
-		if ((ntohs(ippacket->frag_off) & 0x3fff) != 0) {
-		    br =
-			processfragment(ippacket, &sport, &dport,
-					&firstin);
-		    if (!firstin)
-			continue;
-		} else {
-		    sport = transpacket->source;
-		    dport = transpacket->dest;
-		    br = nbplen;
-		}
-
-		/*
-		 * Apply TCP filter, if active.
-		 */
-
-		if ((ofilter->tfiltercode) &&
-		    (!(utfilter(&(ofilter->tfl), ippacket->saddr, ippacket->daddr,
-				ntohs(sport), ntohs(dport), IPPROTO_TCP)))) {
-		    show_ip_stats(statwin, iptotal, tcptotal, udptotal,
-				  icmptotal, noniptotal);
-		    goto cont;
-		}
-		tcpentry =
-		    in_table(&table, ippacket->saddr, ippacket->daddr,
-			     ntohs(sport), ntohs(dport), ifname, tcptm,
-			     logging, logfile, &nomem);
+		tcpentry = in_table(&table, ippacket->saddr, ippacket->daddr,
+			     ntohs(sport), ntohs(dport), ifname,
+			     logging, logfile, &nomem, options);
 
 		/* 
 		 * Add a new entry if it doesn't exist, and, 
@@ -968,6 +1041,7 @@ void ipmon(const struct OPTIONS *options,
 			 */
 
 			if (!nomem) {
+			    wasempty = (table.head == NULL);
 			    tcpentry = addentry(&table,
 						(unsigned long)
 						ippacket->saddr,
@@ -978,10 +1052,28 @@ void ipmon(const struct OPTIONS *options,
 						rvnfd, options->servnames,
 						&nomem);
 
-			    if (tcpentry != NULL)
+			    if (tcpentry != NULL) {
 				printentry(&table,
 					   tcpentry->oth_connection,
 					   screen_idx, mode);
+					   
+				if (wasempty) {
+				    set_barptr((char **) &(table.barptr),
+				        (char *) table.firstvisible,
+				        &(table.firstvisible->starttime),
+				        (char *) &(table.firstvisible->spanbr),
+				        sizeof(unsigned long), statwin, &statcleared, statx);
+				    table.baridx = 1;
+				}
+				
+				if ((table.barptr == tcpentry) ||
+				    (table.barptr == tcpentry->oth_connection))
+				    set_barptr((char **) &(table.barptr),
+				        (char *) table.barptr,
+				        &(table.barptr->starttime),
+				        (char *) &(table.barptr->spanbr),
+				        sizeof(unsigned long), statwin, &statcleared, statx);
+			    }
 			}
 		    }
 		}
@@ -1004,9 +1096,9 @@ void ipmon(const struct OPTIONS *options,
 			    p_dstat = tcpentry->d_fstat;
 			}
 			updateentry(&table, tcpentry, transpacket, tpacket,
-			            linktype, nbplen,
+			            linktype, readlen,
 				    br, ippacket->frag_off, logging,
-				    options->mac, &revlook, rvnfd, logfile,
+				    &revlook, rvnfd, options, logfile,
 				    &nomem);
 
 			/*
@@ -1022,7 +1114,7 @@ void ipmon(const struct OPTIONS *options,
 			        strcat(msgstring, " (SYN)");
 
 			    writetcplog(logging, logfile,
-				    tcpentry, nbplen, options->mac, msgstring);
+				    tcpentry, readlen, options->mac, msgstring);
 		        }
 			
 			if ((revlook)
@@ -1062,10 +1154,8 @@ void ipmon(const struct OPTIONS *options,
 		}
 	    } else {		/* now for the other IP protocols */
 		fragment = ((ntohs(ippacket->frag_off) & 0x1fff) != 0);
-
-		if (ippacket->protocol == IPPROTO_UDP)
-		    udptotal += nbplen;
-		else if (ippacket->protocol == IPPROTO_ICMP) {
+		
+		if (ippacket->protocol == IPPROTO_ICMP) {
 
 		    /*
 		     * Cancel the corresponding TCP entry if an ICMP
@@ -1074,101 +1164,23 @@ void ipmon(const struct OPTIONS *options,
 		     */
 
 		    if (((struct icmphdr *) transpacket)->type ==
-			ICMP_DEST_UNREACH) process_dest_unreach(&table,
-								(char *)
-								transpacket,
-								ifname,
-								&nomem);
+		        ICMP_DEST_UNREACH)
+	                process_dest_unreach(&table, (char *) transpacket,
+	                    ifname, &nomem);
 
-		    icmptotal += nbplen;
-		} else
-		    othptotal += nbplen;
+		}
 
-		/*
-		 * Add and display non-TCP packet entry if memory is ok,
-		 * and if filter permits
-		 */
-
-		if ((!nomem) && (othfilterok(ofilter, ippacket->protocol,
-					     ippacket->saddr,
-					     ippacket->daddr,
-					     ntohs(((struct udphdr *)
-						    transpacket)->source),
-					     ntohs(((struct udphdr *)
-						    transpacket)->dest)))) {
 		    othpent =
 			add_othp_entry(&othptbl, &table, ippacket->saddr,
 				       ippacket->daddr, IS_IP,
 				       ippacket->protocol, linktype,
 				       (char *) tpacket, (char *) transpacket,
-				       nbplen, ifname, &revlook,
-				       rvnfd, tcptm, logging, logfile,
+				       readlen, ifname, &revlook,
+				       rvnfd, options->timeout, logging, logfile,
 				       options->servnames, fragment,
 				       &nomem);
 		}
 	    }
-
-	  cont:
-	    show_ip_stats(statwin, iptotal, tcptotal, udptotal, icmptotal,
-			  noniptotal);
-	}
-	gettimeofday(&tv, NULL);
-	now = tv.tv_sec;
-	unow = tv.tv_sec * 1e+06 + tv.tv_usec;
-        
-        /* 
-         * Print timer at bottom of screen
-         */
-         
-	if (now - timeint >= 5) {
-	    printelapsedtime(starttime, now, othptbl.obmaxy - 1, 15,
-			     othptbl.borderwin);
-	    timeint = now;
-	}
-
-	/*
-	 * Automatically clear closed/timed out entries
-	 */
-
-	if ((options->closedint != 0) &&
-	    ((now - closedint) / 60 >= options->closedint)) {
-	    flushclosedentries(&table, tcptm, &screen_idx,
-			       logging, logfile);
-	    refreshtcpwin(&table, screen_idx, mode);
-	    closedint = now;
-	}
-
-	/*
-	 * Update screen at configured intervals.
-	 */
-
-	if (((options->updrate != 0)
-	     && (now - updtime >= options->updrate))
-	    || ((options->updrate == 0)
-		&& (unow - updtime_usec >= DEFAULT_UPDATE_DELAY))) {
-	    update_panels();
-	    doupdate();
-	    updtime = now;
-	    updtime_usec = unow;
-	}
-	
-	/*
-	 * Terminate facility should a lifetime be specified at the
-	 * command line
-	 */
-	if ((facilitytime != 0)
-	    && (((now - starttime) / 60) >= facilitytime))
-	    exitloop = 1;
-
-        /*
-         * Close and rotate log file if signal was received
-         */
-	if ((rotate_flag == 1) && (logging)) {
-	    announce_rotate_prepare(logfile);
-	    write_tcp_unclosed(logging, logfile, &table);
-	    rotate_logfile(&logfile, target_logname);
-	    announce_rotate_complete(logfile);
-	    rotate_flag = 0;
 	}
     }
 
@@ -1179,7 +1191,6 @@ void ipmon(const struct OPTIONS *options,
 	endservent();
 
     close_rvn_socket(rvnfd);
-    close(isdn_fd);
 
     if ((options->promisc) && (is_last_instance())) {
 	load_promisc_list(&promisc_list);
@@ -1206,8 +1217,7 @@ void ipmon(const struct OPTIONS *options,
     close(fd);
     destroytcptable(&table);
     destroyothptable(&othptbl);
-    destroyfraglist();
-    destroy_isdn_table(&isdntable);
+    pkt_cleanup();
     writelog(logging, logfile,
 	     "******** IP traffic monitor stopped ********\n");
     unmark_facility(IPMONIDFILE, ifptr);
