@@ -28,13 +28,15 @@ details.
 #include <fcntl.h>
 #include <net/if_arp.h>
 #include <ctype.h>
+#include <winops.h>
+#include <menurt.h>
+#include <input.h>
+#include <labels.h>
+#include <msgboxes.h>
+#include <listbox.h>
 #include "deskman.h"
-#include "stdwinset.h"
-#include "menurt.h"
 #include "attrs.h"
 #include "dirs.h"
-#include "input.h"
-#include "error.h"
 #include "landesc.h"
 #include "links.h"
 
@@ -42,7 +44,7 @@ void etherr(void)
 {
     int resp;
 
-    errbox("Unable to open host description file", ANYKEY_MSG, &resp);
+    tx_errbox("Unable to open host description file", ANYKEY_MSG, &resp);
 }
 
 void add_desclist_node(struct desclist *list, struct desclistent *ptmp)
@@ -67,7 +69,7 @@ void add_desclist_node(struct desclist *list, struct desclistent *ptmp)
  * the IPTraf definition files take precedence.
  */
  
-void loaddesclist(struct desclist *list, unsigned int linktype)
+void loaddesclist(struct desclist *list, unsigned int linktype, int withethers)
 {
     struct desclistent *ptmp = NULL;
     FILE *fd = NULL;
@@ -124,6 +126,10 @@ void loaddesclist(struct desclist *list, unsigned int linktype)
      * Author's note:
      *  Moved significantly repeating code to a function.
      */
+
+    if (!withethers)
+        return;
+
     if (linktype != LINK_ETHERNET)
         return;
             
@@ -323,41 +329,36 @@ void operate_descselect(struct desclist *list, struct desclistent **node,
 void selectdesc(struct desclist *list, struct desclistent **node,
 		int *aborted)
 {
-    WINDOW *bw;
-    PANEL *bp;
-    WINDOW *win;
-    PANEL *panel;
     int resp;
-
+    struct scroll_list slist;
+    char descline[80];
+    
     if (list->head == NULL) {
-	errbox("No descriptions", ANYKEY_MSG, &resp);
+	tx_errbox("No descriptions", ANYKEY_MSG, &resp);
 	return;
     }
-    bw = newwin(20, COLS, 2, 0);
-    bp = new_panel(bw);
-    wattrset(bw, BOXATTR);
-    box(bw, ACS_VLINE, ACS_HLINE);
 
-    win = newwin(18, COLS - 2, 3, 1);
-    panel = new_panel(win);
-    stdwinset(win);
-    wattrset(win, STDATTR);
-    colorwin(win);
+    *node = list->head;
+    tx_init_listbox(&slist, COLS, 20, 0, (LINES - 20) / 2,
+        STDATTR, BOXATTR, BARSTDATTR, HIGHATTR);
+    
+    tx_set_listbox_title(&slist, "Address", 1);
+    tx_set_listbox_title(&slist, "Description", 19);
+        
+    while (*node != NULL) {
+        snprintf(descline, 80, "%-18s%s", (*node)->rec.address, (*node)->rec.desc);    
+        tx_add_list_entry(&slist, (char *) (*node), descline);
+        (*node) = (*node)->next_entry;
+    }
+    
+    tx_show_listbox(&slist);
+    tx_operate_listbox(&slist, &resp, aborted);
 
-    listkeyhelp();
+    if (!(*aborted))
+        *node = (struct desclistent *) slist.textptr->nodeptr;
 
-    wmove(bw, 0, 2);
-    wprintw(bw, " MAC Address ");
-    wmove(bw, 0, 18);
-    wprintw(bw, " Address Description ");
-
-    displayethdescs(list, win);
-    operate_descselect(list, node, win, aborted);
-
-    del_panel(panel);
-    delwin(win);
-    del_panel(bp);
-    delwin(bw);
+    tx_close_listbox(&slist);
+    tx_destroy_list(&slist);    
 
     update_panels();
     doupdate();
@@ -374,7 +375,7 @@ void descdlg(struct descrec *rec, char *initaddr, char *initdesc,
     panel = new_panel(win);
 
     wattrset(win, DLGBOXATTR);
-    colorwin(win);
+    tx_colorwin(win);
     box(win, ACS_VLINE, ACS_HLINE);
     wmove(win, 6, 2 * COLS / 80);
     tabkeyhelp(win);
@@ -386,17 +387,19 @@ void descdlg(struct descrec *rec, char *initaddr, char *initdesc,
     wprintw(win, "MAC Address:");
     wmove(win, 4, 2 * COLS / 80);
     wprintw(win, "Description:");
-    initfields(&fieldlist, 3, 52, 10, (COLS - 52) / 2 + 6 * COLS / 80);
-    addfield(&fieldlist, 12, 0, 0, initaddr);
-    addfield(&fieldlist, 50, 2, 0, initdesc);
+    
+    tx_initfields(&fieldlist, 3, 52, 10, (COLS - 52) / 2 + 6 * COLS / 80,
+        DLGTEXTATTR, FIELDATTR);
+    tx_addfield(&fieldlist, 12, 0, 0, initaddr);
+    tx_addfield(&fieldlist, 50, 2, 0, initdesc);
 
-    fillfields(&fieldlist, aborted);
+    tx_fillfields(&fieldlist, aborted);
 
     if (!(*aborted)) {
 	strcpy(rec->address, fieldlist.list->buf);
 	strcpy(rec->desc, fieldlist.list->nextfield->buf);
     }
-    destroyfields(&fieldlist);
+    tx_destroyfields(&fieldlist);
     del_panel(panel);
     delwin(win);
 }
@@ -468,20 +471,22 @@ void ethdescmgr(unsigned int linktype)
     int aborted;
     struct desclist list;
 
-    loaddesclist(&list, linktype);
+    loaddesclist(&list, linktype, WITHOUTETCETHERS);
 
-    initmenu(&menu, 6, 31, (LINES - 6) / 2, (COLS - 31) / 2);
-    additem(&menu, " ^A^dd description...",
-	    "Adds a description for an Ethernet address");
-    additem(&menu, " ^E^dit description...",
-	    "Modifies an existing Ethernet address description");
-    additem(&menu, " ^D^elete description...",
-	    "Deletes an existing Ethernet address description");
-    additem(&menu, " E^x^it menu", "Returns to the main menu");
+    tx_initmenu(&menu, 7, 31, (LINES - 6) / 2, (COLS - 31) / 2,
+        BOXATTR, STDATTR, HIGHATTR, BARSTDATTR, BARHIGHATTR, DESCATTR);
+    tx_additem(&menu, " ^A^dd description...",
+	    "Adds a description for a MAC address");
+    tx_additem(&menu, " ^E^dit description...",
+	    "Modifies an existing MAC address description");
+    tx_additem(&menu, " ^D^elete description...",
+	    "Deletes an existing MAC address description");
+    tx_additem(&menu, NULL, NULL);
+    tx_additem(&menu, " E^x^it menu", "Returns to the main menu");
 
     do {
-	showmenu(&menu);
-	operatemenu(&menu, &row, &aborted);
+	tx_showmenu(&menu);
+	tx_operatemenu(&menu, &row, &aborted);
 
 	switch (row) {
 	case 1:
@@ -494,9 +499,9 @@ void ethdescmgr(unsigned int linktype)
 	    delethdesc(&list);
 	    break;
 	}
-    } while (row != 4);
+    } while (row != 5);
 
-    destroymenu(&menu);
+    tx_destroymenu(&menu);
     update_panels();
     doupdate();
     savedesclist(&list, linktype);
