@@ -18,8 +18,11 @@ GNU General Public License for more details.
 
 #include "descrec.h"
 
+#define WITHETCETHERS 1
+#define WITHOUTETCETHERS 0
+
 void etherr(void);
-void loaddesclist(struct desclist *list, unsigned int linktype);
+void loaddesclist(struct desclist *list, unsigned int linktype, int withethers);
 void savedesclist(struct desclist *list, unsigned int linktype);
 void displayethdescs(struct desclist *list, WINDOW * win);
 void destroydesclist(struct desclist *list);
