@@ -24,10 +24,12 @@ details.
 #include <netinet/in.h>
 #include <linux/if_ether.h>
 #include <net/if_arp.h>
+#include <input.h>
+#include <msgboxes.h>
 #include "attrs.h"
 #include "deskman.h"
 #include "dirs.h"
-#include "error.h"
+#include "options.h"
 #include "tcptable.h"
 #include "othptab.h"
 #include "ifstats.h"
@@ -36,8 +38,6 @@ details.
 #include "hostmon.h"
 #include "links.h"
 #include "mode.h"
-#include "options.h"
-#include "input.h"
 
 #define MSGSTRING_MAX		240
 #define TARGET_LOGNAME_MAX	160
@@ -50,7 +50,7 @@ char current_logfile[TARGET_LOGNAME_MAX];
 void openlogerr()
 {
     int resp;
-    errbox("Unable to open log file", ANYKEY_MSG, &resp);
+    tx_errbox("Unable to open log file", ANYKEY_MSG, &resp);
 }
 
 /*
@@ -77,7 +77,7 @@ void input_logfile(char *target, int *logging)
     dlgpanel = new_panel(dlgwin);
     
     wattrset(dlgwin, DLGBOXATTR);
-    colorwin(dlgwin);
+    tx_colorwin(dlgwin);
     box(dlgwin, ACS_VLINE, ACS_HLINE);
     mvwprintw(dlgwin, 0, 1, " Logging Enabled ");
     wattrset(dlgwin, DLGTEXTATTR);
@@ -88,9 +88,10 @@ void input_logfile(char *target, int *logging)
     stdkeyhelp(dlgwin);
     wprintw(dlgwin, " (turns logging off)");
     
-    initfields(&fieldlist, 1, 50, (LINES - 1) / 2 + 2, (COLS - 50) / 2 - 3);
-    addfield(&fieldlist, 48, 0, 0, target);
-    fillfields(&fieldlist, &aborted);
+    tx_initfields(&fieldlist, 1, 50, (LINES - 1) / 2 + 2, (COLS - 50) / 2 - 3,
+        DLGTEXTATTR, FIELDATTR);
+    tx_addfield(&fieldlist, 48, 0, 0, target);
+    tx_fillfields(&fieldlist, &aborted);
     
     if (!aborted) {
         if (strchr(fieldlist.list->buf, '/') == NULL)
@@ -101,7 +102,7 @@ void input_logfile(char *target, int *logging)
     
     *logging = !aborted;
         
-    destroyfields(&fieldlist);
+    tx_destroyfields(&fieldlist);
     del_panel(dlgpanel);
     delwin(dlgwin);
     update_panels();
@@ -238,23 +239,6 @@ void writeothplog(int logging, FILE * fd, char *protname,
     }
 }
 
-void write_timeout_log(int logging, FILE * logfile,
-		       struct tcptableent *tcpnode)
-{
-    char msgstring[MSGSTRING_MAX];
-
-    if (logging) {
-	sprintf(msgstring,
-		"TCP; Connection %s:%s to %s:%s timed out, %lu packets, %lu bytes; opposite direction %lu packets, %lu bytes",
-		tcpnode->s_fqdn, tcpnode->s_sname,
-		tcpnode->d_fqdn, tcpnode->d_sname,
-		tcpnode->pcount, tcpnode->bcount,
-		tcpnode->oth_connection->pcount,
-		tcpnode->oth_connection->bcount);
-	writelog(logging, logfile, msgstring);
-    }
-}
-
 void writegstatlog(struct iftab *table, int unit, unsigned long nsecs,
 		   FILE * fd)
 {
@@ -359,6 +343,7 @@ void writedstatlog(char *ifname, int unit, float activity, float pps,
 	    "\t(incoming: %llu packets, %llu bytes; outgoing: %llu packets, %llu bytes)\n",
 	    ts->noniptotal_in, ts->nonipbtotal_in, ts->noniptotal_out,
 	    ts->nonipbtotal_out);
+    fprintf(fd, "Broadcast: %llu packets, %llu bytes\n", ts->bcast, ts->bcastbytes);
 
     if (nsecs > 5) {
 	fprintf(fd, "\nAverage rates:\n");
@@ -399,28 +384,60 @@ void writedstatlog(char *ifname, int unit, float activity, float pps,
     fflush(fd);
 }
 
-void writeutslog(struct portlistent *list, unsigned long nsecs, FILE * fd)
+void writeutslog(struct portlistent *list, unsigned long nsecs, int units,
+    FILE * fd)
 {
     char atime[TIME_TARGET_MAX];
     struct portlistent *ptmp = list;
-
+    char unitstring[10];
+    float inrate, outrate, totalrate;
+    time_t now = time(NULL);
+        
+    dispmode(units, unitstring);
+    
     genatime(time((time_t *) NULL), atime);
 
     fprintf(fd, "\n*** TCP/UDP traffic log, generated %s\n\n", atime);
 
     while (ptmp != NULL) {
+        if (now - ptmp->proto_starttime < 5)
+            inrate = outrate = totalrate = -1.0;
+        else {
+            if (units == KBITS) {
+                inrate = (float) (ptmp->ibcount * 8 / 1000) / (float) (now - ptmp->proto_starttime);
+                outrate = (float) (ptmp->obcount * 8 / 1000) / (float) (now - ptmp->proto_starttime);
+                totalrate = (float) (ptmp->bcount * 8 / 1000) / (float) (now - ptmp->proto_starttime);
+            } else {
+                inrate = (float) (ptmp->obcount / 1024) / (float) (now - ptmp->proto_starttime);
+                outrate = (float) (ptmp->obcount / 1024) / (float) (now - ptmp->proto_starttime);
+                totalrate = (float) (ptmp->obcount / 1024) / (float) (now - ptmp->proto_starttime);
+            }
+        }
+        
 	if (ptmp->protocol == IPPROTO_TCP)
 	    fprintf(fd, "TCP/%s: ", ptmp->servname);
 	else
 	    fprintf(fd, "UDP/%s: ", ptmp->servname);
 
-	fprintf(fd, "%llu packets, %llu bytes total; ", ptmp->count,
+	fprintf(fd, "%llu packets, %llu bytes total", ptmp->count,
 		ptmp->bcount);
-	fprintf(fd, "%llu packets, %llu bytes incoming; ", ptmp->icount,
+
+        if (totalrate >= 0.0)
+            fprintf(fd, ", %.2f %s/s", totalrate, unitstring);
+            
+	fprintf(fd, "; %llu packets, %llu bytes incoming", ptmp->icount,
 		ptmp->ibcount);
-	fprintf(fd, "%llu packets, %llu bytes outgoing\n\n", ptmp->ocount,
+
+        if (inrate >= 0.0)
+            fprintf(fd, ", %.2f %s/s", inrate, unitstring);
+            
+	fprintf(fd, "; %llu packets, %llu bytes outgoing", ptmp->ocount,
 		ptmp->obcount);
 
+        if (outrate >= 0.0)
+            fprintf(fd, ", %.2f %s/s", outrate, unitstring);
+            
+        fprintf(fd, "\n\n");
 	ptmp = ptmp->next_entry;
     }
 
