@@ -18,11 +18,13 @@ details.
 
 #include <asm/types.h>
 #include <linux/if_ether.h>
+#include <linux/if_tr.h>
 #include <linux/if_fddi.h>
+#include <winops.h>
 #include "arphdr.h"
+#include "options.h"
 #include "tcptable.h"
 #include "othptab.h"
-#include "stdwinset.h"
 #include "deskman.h"
 #include "attrs.h"
 #include "log.h"
@@ -62,11 +64,11 @@ void init_othp_table(struct othptable *table, int mac)
     table->othpwin = newwin(winht - 2, COLS - 2, wintop + 1, 1);
     table->othppanel = new_panel(table->othpwin);
     wattrset(table->othpwin, STDATTR);
-    colorwin(table->othpwin);
+    tx_colorwin(table->othpwin);
     update_panels();
     doupdate();
 
-    stdwinset(table->othpwin);
+    tx_stdwinset(table->othpwin);
     getmaxyx(table->borderwin, table->obmaxy, obmaxx);
     table->oimaxy = table->obmaxy - 2;
     
@@ -88,13 +90,15 @@ void process_dest_unreach(struct tcptable *table, char *packet,
     tcp = (struct tcphdr *) (packet + 8 + (ip->ihl * 4));
 
     /* 
-     * We really won't be making use of nomem here
+     * We really won't be making use of nomem here.  Timeout checking
+     * won't be performed either, so we just pass NULL as the pointer
+     * to the configuration structure.  in_table() will recognize this
+     * and set its internal timeout variable to 0.
      */
 
     tcpentry = in_table(table, ip->saddr, ip->daddr,
 			ntohs(tcp->source), ntohs(tcp->dest), ifname,
-			0, 0, NULL, nomem);	/* we'll not do any timeout */
-    /* checking here */
+			0, NULL, nomem, NULL);
 
     if (tcpentry != NULL) {
 	tcpentry->stat = tcpentry->oth_connection->stat = FLAG_RST;
@@ -140,7 +144,12 @@ struct othptabent *add_othp_entry(struct othptable *table,
 			new_entry->smacaddr);
 	    convmacaddr(((struct fddihdr *) packet)->daddr,
 			new_entry->dmacaddr);
-        }
+        } else if (linkproto == LINK_TR) {
+            convmacaddr(((struct trh_hdr *) packet)->saddr,
+                        new_entry->smacaddr);
+	    convmacaddr(((struct trh_hdr *) packet)->daddr,
+			new_entry->dmacaddr);
+	}
     }
         
     if (is_ip) {
@@ -589,7 +598,7 @@ void refresh_othwindow(struct othptable *table)
     struct othptabent *entry;
 
     wattrset(table->othpwin, STDATTR);
-    colorwin(table->othpwin);
+    tx_colorwin(table->othpwin);
 
     entry = table->firstvisible;
 
