@@ -137,7 +137,7 @@ struct othptabent *add_othp_entry(struct othptable *table,
 				  int protocol, unsigned short linkproto,
 				  char *packet,char *netpacket,
 				  unsigned int br, char *ifname,
-				  int rev_lookup, int rvnamedon, 
+				  int *rev_lookup, int rvnamedon, 
 				  unsigned int tm, int logging,
 				  FILE * logfile, int servnames, int fragment,
 				  int *nomem);
