@@ -27,6 +27,7 @@ details.
 #include <netinet/in.h>
 #include <sys/un.h>
 #include <arpa/inet.h>
+#include <time.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -43,7 +44,7 @@ char *gen_unix_sockname(void)
     static char scratch[80];
     
     srandom(time(NULL));
-    snprintf(scratch, 80, "%s-%lu%d%u", SOCKET_PREFIX,
+    snprintf(scratch, 80, "%s-%lu%d%ld", SOCKET_PREFIX,
                                         time(NULL), getpid(), random()); 
     
     return scratch;
