@@ -22,6 +22,7 @@ details.
 #include <unistd.h>
 #include <fcntl.h>
 #include <signal.h>
+#include <time.h>
 #include <sys/time.h>
 #include <netdb.h>
 #include <ctype.h>
@@ -33,20 +34,25 @@ details.
 #include <netinet/udp.h>
 #include <linux/if_packet.h>
 #include <linux/if_ether.h>
+#include <listbox.h>
+#include <labels.h>
+#include <winops.h>
+#include <input.h>
+#include <msgboxes.h>
 #include "tcphdr.h"
 #include "dirs.h"
 #include "ipcsum.h"
 #include "deskman.h"
 #include "isdntab.h"
+#include "fltdefs.h"
+#include "utfilter.h"
+#include "fltselect.h"
 #include "packet.h"
 #include "ipfrag.h"
 #include "ifaces.h"
-#include "stdwinset.h"
-#include "input.h"
 #include "attrs.h"
 #include "serv.h"
 #include "servname.h"
-#include "error.h"
 #include "log.h"
 #include "timer.h"
 #include "promisc.h"
@@ -54,6 +60,9 @@ details.
 #include "instances.h"
 #include "packet.h"
 #include "logvars.h"
+#include "error.h"
+#include "bar.h"
+#include "mode.h"
 
 #define SCROLLUP 0
 #define SCROLLDOWN 1
@@ -65,7 +74,7 @@ extern int exitloop;
 extern int daemonized;
 
 extern void writeutslog(struct portlistent *list, unsigned long nsecs,
-			FILE * logfile);
+			int unit, FILE * logfile);
 
 /*
  * SIGUSR1 logfile rotation signal handler
@@ -81,11 +90,14 @@ void rotate_serv_log()
 void initportlist(struct portlist *list)
 {
     float screen_scale = ((float) COLS / 80 + 1) / 2;
-    list->head = list->tail = NULL;
+    int scratchx;
+    
+    list->head = list->tail = list->barptr = NULL;
     list->firstvisible = list->lastvisible = NULL;
     list->count = 0;
-
-    list->borderwin = newwin(LINES - 2, COLS, 1, 0);
+    list->baridx = 0;
+    
+    list->borderwin = newwin(LINES - 3, COLS, 1, 0);
     list->borderpanel = new_panel(list->borderwin);
     wattrset(list->borderwin, BOXATTR);
     box(list->borderwin, ACS_VLINE, ACS_HLINE);
@@ -105,13 +117,14 @@ void initportlist(struct portlist *list)
     wmove(list->borderwin, 0, 67 * screen_scale);
     wprintw(list->borderwin, " BytesFrom ");
 
-    list->win = newwin(LINES - 4, COLS - 2, 2, 1);
+    list->win = newwin(LINES - 5, COLS - 2, 2, 1);
     list->panel = new_panel(list->win);
+    getmaxyx(list->win, list->imaxy, scratchx);
 
-    stdwinset(list->win);
+    tx_stdwinset(list->win);
     wtimeout(list->win, -1);
     wattrset(list->win, STDATTR);
-    colorwin(list->win);
+    tx_colorwin(list->win);
     update_panels();
     doupdate();
 }
@@ -134,6 +147,7 @@ struct portlistent *addtoportlist(struct portlist *list,
 	list->head = ptemp;
 	list->firstvisible = ptemp;
     }
+    
     if (list->tail != NULL) {
 	list->tail->next_entry = ptemp;
 	ptemp->prev_entry = list->tail;
@@ -157,10 +171,12 @@ struct portlistent *addtoportlist(struct portlist *list,
     list->count++;
     ptemp->idx = list->count;
 
-    if (list->count <= LINES - 4)
+    ptemp->proto_starttime = time(NULL);
+    
+    if (list->count <= LINES - 5)
 	list->lastvisible = ptemp;
 
-    wmove(list->borderwin, LINES - 3, 1);
+    wmove(list->borderwin, LINES - 4, 1);
     wprintw(list->borderwin, " %u entries ", list->count);
 
     return ptemp;
@@ -206,23 +222,43 @@ void printportent(struct portlist *list, struct portlistent *entry,
 {
     unsigned int target_row;
     float screen_scale = ((float) COLS / 80 + 1) / 2;
-
-    if ((entry->idx < idx) || (entry->idx > idx + (LINES - 5)))
+    int tcplabelattr;
+    int udplabelattr;
+    int highattr;
+    char sp_buf[10];
+    
+    if ((entry->idx < idx) || (entry->idx > idx + (LINES - 6)))
 	return;
 
     target_row = entry->idx - idx;
     
+    if (entry == list->barptr) {
+        tcplabelattr = BARSTDATTR;
+        udplabelattr = BARPTRATTR;
+        highattr = BARHIGHATTR;
+    } else {
+        tcplabelattr = STDATTR;
+        udplabelattr = PTRATTR;
+        highattr = HIGHATTR;
+    }
+    
+    wattrset(list->win, tcplabelattr);
+    sprintf(sp_buf, "%%%dc", COLS - 2);
+    scrollok(list->win, 0);
+    mvwprintw(list->win, target_row, 0, sp_buf, ' ');
+    scrollok(list->win, 1);
+    
     wmove(list->win, target_row, 1);
     if (entry->protocol == IPPROTO_TCP) {
-	wattrset(list->win, STDATTR);
+	wattrset(list->win, tcplabelattr);
 	wprintw(list->win, "TCP");
     } else if (entry->protocol == IPPROTO_UDP) {
-	wattrset(list->win, PTRATTR);
+	wattrset(list->win, udplabelattr);
 	wprintw(list->win, "UDP");
     }
     
-    wprintw(list->win, "/%s:         ", entry->servname);
-    wattrset(list->win, HIGHATTR);
+    wprintw(list->win, "/%s          ", entry->servname);
+    wattrset(list->win, highattr);
     wmove(list->win, target_row, 17 * screen_scale);
     printlargenum(entry->count, list->win);
     wmove(list->win, target_row, 27 * screen_scale);
@@ -273,12 +309,15 @@ void printport(struct portlist *list, unsigned int protocol,
 
 	listent->count++;
 	listent->bcount += br;
-
+        listent->spans.spanbr += br;
+        
 	if (fromto == 0) {	/* from */
 	    listent->obcount += br;
+	    listent->spans.spanbr_out += br;
 	    listent->ocount++;
 	} else {		/* to */
 	    listent->ibcount += br;
+	    listent->spans.spanbr_in += br;
 	    listent->icount++;
 	}
 
@@ -365,6 +404,26 @@ unsigned long long qp_getkey(struct portlistent *entry, int ch)
     return result;
 }
 
+/*
+ * Refresh TCP/UDP service screen.
+ */
+ 
+void refresh_serv_screen(struct portlist *table, int idx)
+{
+    struct portlistent *ptmp = table->firstvisible;
+
+    wattrset(table->win, STDATTR);    
+    tx_colorwin(table->win);
+    
+    while ((ptmp != NULL) && (ptmp->prev_entry != table->lastvisible)) {
+        printportent(table, ptmp, idx);
+        ptmp = ptmp->next_entry;
+    }
+    update_panels();
+    doupdate();
+}
+      
+
 /*
  * Compare the sort criterion with the pivot value.  Receives a parameter
  * specifying whether the criterion is left or right of the pivot value.
@@ -480,7 +539,7 @@ void sortportents(struct portlist *list, int *idx, int command)
     struct portlistent *ptemp1;
     unsigned int idxtmp;
     
-    if (!list->head)
+    if (!(list->head))
         return;
         
     command = toupper(command);
@@ -494,12 +553,14 @@ void sortportents(struct portlist *list, int *idx, int command)
     
     ptemp1 = list->firstvisible = list->head;
     *idx = 1;
-    idxtmp = 0;
-    while ((ptemp1) && (idxtmp <= LINES - 4)) {	/* printout */
+    idxtmp = 1;
+    
+    while ((ptemp1) && (idxtmp <= LINES - 5)) {	/* printout */
 	printportent(list, ptemp1, *idx);
-	if (++idxtmp <= LINES - 4)
+	if (idxtmp <= LINES - 5)
 	    list->lastvisible = ptemp1;
 	ptemp1 = ptemp1->next_entry;
+	idxtmp++;
     }
 }
 
@@ -514,7 +575,7 @@ void scrollservwin(struct portlist *table, int direction, int *idx)
 	    table->lastvisible = table->lastvisible->next_entry;
 	    table->firstvisible = table->firstvisible->next_entry;
 	    (*idx)++;
-	    wmove(table->win, LINES - 5, 0);
+	    wmove(table->win, LINES - 6, 0);
 	    scrollok(table->win, 0);
 	    wprintw(table->win, sp_buf, ' ');
 	    scrollok(table->win, 1);
@@ -538,16 +599,21 @@ void pageservwin(struct portlist *table, int direction, int *idx)
     int i = 1;
 
     if (direction == SCROLLUP) {
-	while ((i <= LINES - 7) && (table->lastvisible != table->tail)) {
+	while ((i <= LINES - 9) && (table->lastvisible != table->tail)) {
 	    i++;
-	    scrollservwin(table, direction, idx);
+	    table->firstvisible = table->firstvisible->next_entry;
+	    table->lastvisible = table->lastvisible->next_entry;
+	    (*idx)++;
 	}
     } else {
-	while ((i <= LINES - 7) && (table->firstvisible != table->head)) {
+	while ((i <= LINES - 9) && (table->firstvisible != table->head)) {
 	    i++;
-	    scrollservwin(table, direction, idx);
+            table->firstvisible = table->firstvisible->prev_entry;
+            table->lastvisible = table->lastvisible->prev_entry;
+            (*idx)--;
 	}
     }
+    refresh_serv_screen(table, *idx);
 }
 
 void show_portsort_keywin(WINDOW ** win, PANEL ** panel)
@@ -556,50 +622,79 @@ void show_portsort_keywin(WINDOW ** win, PANEL ** panel)
     *panel = new_panel(*win);
 
     wattrset(*win, DLGBOXATTR);
-    colorwin(*win);
+    tx_colorwin(*win);
     box(*win, ACS_VLINE, ACS_HLINE);
 
     wattrset(*win, DLGTEXTATTR);
-    mvwprintw(*win, 2, 2, "Select display field to sort by");
+    mvwprintw(*win, 2, 2, "Select sort criterion");
     wmove(*win, 4, 2);
-    printkeyhelp("R", " - port number", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("R", " - port number", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 5, 2);
-    printkeyhelp("P", " - total packets", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("P", " - total packets", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 6, 2);
-    printkeyhelp("B", " - total bytes", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("B", " - total bytes", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 7, 2);
-    printkeyhelp("T", " - packets to", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("T", " - packets to", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 8, 2);
-    printkeyhelp("O", " - bytes to", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("O", " - bytes to", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 9, 2);
-    printkeyhelp("F", " - packets from", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("F", " - packets from", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 10, 2);
-    printkeyhelp("M", " - bytes from", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("M", " - bytes from", *win, DLGHIGHATTR, DLGTEXTATTR);
     wmove(*win, 11, 2);
-    printkeyhelp("Any other key", " - cancel sort", *win, DLGHIGHATTR, DLGTEXTATTR);
+    tx_printkeyhelp("Any other key", " - cancel sort", *win, DLGHIGHATTR, DLGTEXTATTR);
     update_panels();
     doupdate();
 }
 
+void update_serv_rates(struct portlist *list, WINDOW *win, int actmode,
+    int *cleared)
+{
+    char act_unit[10];
+    float inrate, outrate, totalrate;
+    time_t now = time(NULL);
+
+    dispmode(actmode, act_unit);
+            
+    if (actmode == KBITS) {
+        inrate = (float) (list->barptr->spans.spanbr_in * 8 / 1000) / (float) (now - list->barptr->starttime);
+        outrate = (float) (list->barptr->spans.spanbr_out * 8 / 1000) / (float) (now - list->barptr->starttime);
+        totalrate = (float) (list->barptr->spans.spanbr * 8 / 1000) / (float) (now - list->barptr->starttime);
+    } else {
+        inrate = (float) (list->barptr->spans.spanbr_in / 1024) / (float) (now - list->barptr->starttime);
+        outrate = (float) (list->barptr->spans.spanbr_out / 1024) / (float) (now - list->barptr->starttime);
+        totalrate = (float) (list->barptr->spans.spanbr / 1024) / (float) (now - list->barptr->starttime);
+    }    
+        
+    wattrset(win, IPSTATLABELATTR);
+    mvwprintw(win, 0, 1, "Protocol data rates (%s/s):           in             out            total", act_unit);
+    wattrset(win, IPSTATATTR);
+    mvwprintw(win, 0, 31, "%10.2f", inrate);            
+    mvwprintw(win, 0, 46, "%10.2f", outrate);            
+    mvwprintw(win, 0, 61, "%10.2f", totalrate);
+    
+    bzero(&(list->barptr->spans), sizeof(struct serv_spans));
+    list->barptr->starttime = time(NULL);
+    *cleared = 0;            
+}
+
 /*
  * The TCP/UDP service monitor
  */
 
 void servmon(char *ifname, struct porttab *ports,
-	     const struct OPTIONS *options, int facilitytime)
+	     const struct OPTIONS *options, int facilitytime,
+	     struct filterstate *ofilter)
 {
     int logging = options->logging;
     int fd;
-    int isdn_fd = -1;
+    int pkt_result;
 
     char buf[MAX_PACKET_SIZE];
-    char aligned_buf[ALIGNED_BUF_LEN];
     char *ipacket;
-    char *tpacket;
 
     int keymode = 0;
 
-    unsigned int iphlen;
     struct sockaddr_ll fromaddr;
     unsigned short linktype;
     int br;
@@ -609,7 +704,6 @@ void servmon(char *ifname, struct porttab *ports,
 
     unsigned int sport = 0;
     unsigned int dport = 0;
-    unsigned int firstin;
 
     struct timeval tv;
     unsigned long starttime, startlog, timeint;
@@ -618,23 +712,28 @@ void servmon(char *ifname, struct porttab *ports,
     unsigned long updtime = 0;
     unsigned long long updtime_usec = 0;
 
-    int csum;
-    int ck;
+    unsigned int tot_br;
 
     int ch;
 
     struct portlist list;
-
-    struct isdntab isdntable;
-
+    struct portlistent *serv_tmp;
+    int statcleared = 0;
+    
     FILE *logfile = NULL;
 
     struct promisc_states *promisc_list;
 
     WINDOW *sortwin;
     PANEL *sortpanel;
-
+    
+    WINDOW *statwin;
+    PANEL *statpanel;
+    
     char msgstring[80];
+    char sp_buf[10];
+    
+    const int statx = 1;
     
     /*
      * Mark this facility
@@ -676,15 +775,18 @@ void servmon(char *ifname, struct porttab *ports,
     active_facility_countfile[0] = '\0';
     
     initportlist(&list);
-
+    statwin = newwin(1, COLS, LINES - 2, 0);
+    statpanel = new_panel(statwin);
+    scrollok(statwin, 0);
+    wattrset(statwin, IPSTATLABELATTR);
+    sprintf(sp_buf, "%%%dc", COLS);
+    mvwprintw(statwin, 0, 0, sp_buf, ' ');
+        
     move(LINES - 1, 1);
     scrollkeyhelp();
     sortkeyhelp();
     stdexitkeyhelp();
 
-    update_panels();
-    doupdate();
-
     if (options->servnames)
 	setservent(1);
 
@@ -709,12 +811,16 @@ void servmon(char *ifname, struct porttab *ports,
     writelog(logging, logfile,
 	     "******** TCP/UDP service monitor started ********");
 
-    bzero(&isdntable, sizeof(struct isdntab));
-
+    isdnfd = -1;
     exitloop = 0;
     gettimeofday(&tv, NULL);
     starttime = startlog = timeint = tv.tv_sec;
 
+    wattrset(statwin, IPSTATATTR);
+    mvwprintw(statwin, 0, 1, "No entries");
+    update_panels();
+    doupdate();
+    
     while (!exitloop) {
 	getpacket(fd, buf, &fromaddr, &ch, &br, iface, list.win);
 
@@ -722,23 +828,77 @@ void servmon(char *ifname, struct porttab *ports,
 	    if (keymode == 0) {
 		switch (ch) {
 		case KEY_UP:
-		    scrollservwin(&list, SCROLLDOWN, &idx);
+		    if (list.barptr != NULL) {
+		        if (list.barptr->prev_entry != NULL) {
+		            serv_tmp = list.barptr;
+		            set_barptr((char **) &(list.barptr),
+		                (char *) list.barptr->prev_entry,
+		                &(list.barptr->prev_entry->starttime),
+		                (char *) &(list.barptr->prev_entry->spans),
+		                sizeof(struct serv_spans), statwin, &statcleared, statx);
+		            printportent(&list, serv_tmp, idx);
+		            
+		            if (list.baridx == 1) {
+		                scrollservwin(&list, SCROLLDOWN, &idx);
+		            } else
+		                list.baridx--;
+		            
+		            printportent(&list, list.barptr, idx);
+		        }
+		    }
 		    break;
 		case KEY_DOWN:
-		    scrollservwin(&list, SCROLLUP, &idx);
+		    if (list.barptr != NULL) {
+		        if (list.barptr->next_entry != NULL) {
+		            serv_tmp = list.barptr;
+		            set_barptr((char **) &(list.barptr),
+		                (char *) list.barptr->next_entry,
+		                &(list.barptr->next_entry->starttime),
+		                (char *) &(list.barptr->next_entry->spans),
+		                sizeof(struct serv_spans), statwin, &statcleared, statx);
+		            printportent(&list, serv_tmp, idx);
+		            
+		            if (list.baridx == list.imaxy) {
+		                scrollservwin(&list, SCROLLUP, &idx);
+		            } else
+		                list.baridx++;
+		            
+		            printportent(&list, list.barptr, idx);
+		        }
+		    }
 		    break;
 		case KEY_PPAGE:
 		case '-':
-		    pageservwin(&list, SCROLLDOWN, &idx);
+		    if (list.barptr != NULL) {
+		        pageservwin(&list, SCROLLDOWN, &idx);
+		        
+		        set_barptr((char **) &(list.barptr), (char *) (list.lastvisible),
+		            &(list.lastvisible->starttime),
+		            (char *) &(list.lastvisible->spans),
+		            sizeof(struct serv_spans), statwin, &statcleared, statx);
+		        list.baridx = list.lastvisible->idx - idx + 1;
+		        
+		        refresh_serv_screen(&list, idx);
+		    }
 		    break;
 		case KEY_NPAGE:
 		case ' ':
-		    pageservwin(&list, SCROLLUP, &idx);
+		    if (list.barptr != NULL) {
+		        pageservwin(&list, SCROLLUP, &idx);
+		        
+		        set_barptr((char **) &(list.barptr), (char *) (list.firstvisible),
+		            &(list.firstvisible->starttime),
+		            (char *) &(list.firstvisible->spans),
+		            sizeof(struct serv_spans), statwin, &statcleared, statx);
+		        list.baridx = 1;
+		        
+		        refresh_serv_screen(&list, idx);
+		    }
 		    break;
 		case 12:
 		case 'l':
 		case 'L':
-		    refresh_screen();
+		    tx_refresh_screen();
 		    break;
 		case 's':
 		case 'S':
@@ -758,68 +918,45 @@ void servmon(char *ifname, struct porttab *ports,
 		delwin(sortwin);
 		sortportents(&list, &idx, ch);
 		keymode = 0;
+		if (list.barptr != NULL) {
+		    set_barptr((char **) &(list.barptr), (char *) list.firstvisible,
+		        &(list.firstvisible->starttime),
+		        (char *) &(list.firstvisible->spans),
+		        sizeof(struct serv_spans), statwin, &statcleared, statx);
+		    list.baridx = 1;
+		}
+		refresh_serv_screen(&list, idx);
+		update_panels();
+		doupdate();
 	    }
 	}
 
 	if (br > 0) {
-	    if ((fromaddr.sll_protocol == ntohs(ETH_P_IP)) &&
-		((strcmp(iface, ifname) == 0))) {
-
-		isdn_iface_check(&isdn_fd, iface);
-		linktype = getlinktype(fromaddr.sll_hatype, iface,
-				       isdn_fd, &isdntable);
-
-		adjustpacket(buf, linktype, &ipacket, aligned_buf, &br);
-
-		if (ipacket == NULL)
+            pkt_result = processpacket(buf, &ipacket, &br, &tot_br,
+                    &sport, &dport, &fromaddr, &linktype,
+                    ofilter, iface, ifname);
+                    
+            if (pkt_result != PACKET_OK)
 		    continue;
 
-		iphlen = ((struct iphdr *) ipacket)->ihl * 4;
-		ck = ((struct iphdr *) ipacket)->check;
-		((struct iphdr *) ipacket)->check = 0;
-		csum = in_cksum((u_short *) ipacket, iphlen);
-
-		if (ck != csum)
-		    continue;
-
-		/*
-		 * if not first fragment of a datagram, pass it to
-		 * the fragment processor and get the ports from there.
-		 */
-
-		if ((ntohs(((struct iphdr *) ipacket)->frag_off) & 0x3fff)
-		    != 0) {
-		    br =
-			processfragment((struct iphdr *) ipacket, &sport,
-					&dport, &firstin);
-		    if (!firstin)
-			continue;
-		} else {
-		    tpacket = ipacket + iphlen;
-
-		    if (((struct iphdr *) ipacket)->protocol ==
-			IPPROTO_TCP) {
-			sport = ntohs(((struct tcphdr *) tpacket)->source);
-			dport = ntohs(((struct tcphdr *) tpacket)->dest);
-		    } else if (((struct iphdr *) ipacket)->protocol ==
-			       IPPROTO_UDP) {
-			sport = ntohs(((struct udphdr *) tpacket)->source);
-			dport = ntohs(((struct udphdr *) tpacket)->dest);
-		    }
-		}
-
-		if ((((struct iphdr *) ipacket)->protocol == IPPROTO_TCP)
+            if ((((struct iphdr *) ipacket)->protocol == IPPROTO_TCP)
 		    || (((struct iphdr *) ipacket)->protocol ==
 			IPPROTO_UDP)) {
-		    printport(&list, ((struct iphdr *) ipacket)->protocol,
-			      sport,
+		printport(&list, ((struct iphdr *) ipacket)->protocol,
+			      ntohs(sport),
 			      ntohs(((struct iphdr *) ipacket)->tot_len),
 			      idx, 0, ports, options->servnames);
-		    printport(&list, ((struct iphdr *) ipacket)->protocol,
-			      dport,
+		printport(&list, ((struct iphdr *) ipacket)->protocol,
+			      ntohs(dport),
 			      ntohs(((struct iphdr *) ipacket)->tot_len),
 			      idx, 1, ports, options->servnames);
-		}
+                    
+                if ((list.barptr == NULL) && (list.head != NULL)) {
+                    set_barptr((char **) &(list.barptr), (char *) list.head,
+                            &(list.head->starttime), (char *) &(list.head->spans),
+                            sizeof(struct serv_spans), statwin, &statcleared, statx);
+                    list.baridx = 1;
+                }        
 	    }
 	}
 	gettimeofday(&tv, NULL);
@@ -827,14 +964,22 @@ void servmon(char *ifname, struct porttab *ports,
 	unow = tv.tv_sec * 1e+6 + tv.tv_usec;
 
 	if (now - timeint >= 5) {
-	    printelapsedtime(starttime, now, LINES - 3, 20,
+	    printelapsedtime(starttime, now, LINES - 4, 20,
 			     list.borderwin);
 	    timeint = now;
 	}
 	if (((now - startlog) >= options->logspan) && (logging)) {
-	    writeutslog(list.head, now - starttime, logfile);
+	    writeutslog(list.head, now - starttime, options->actmode,
+	        logfile);
 	    startlog = now;
 	}
+        
+        if (list.barptr != NULL) {
+            if ((now - list.barptr->starttime) >= 5) {
+                update_serv_rates(&list, statwin, options->actmode, &statcleared);
+            }
+        }
+        
 	if (((options->updrate != 0)
 	     && (now - updtime >= options->updrate))
 	    || ((options->updrate == 0)
@@ -853,7 +998,8 @@ void servmon(char *ifname, struct porttab *ports,
 
     if (logging) {
 	signal(SIGUSR1, SIG_DFL);
-	writeutslog(list.head, time((time_t *) NULL) - starttime, logfile);
+	writeutslog(list.head, time((time_t *) NULL) - starttime, options->actmode,
+	    logfile);
 	writelog(logging, logfile,
 		 "******** TCP/UDP service monitor stopped ********");
 	fclose(logfile);
@@ -873,12 +1019,13 @@ void servmon(char *ifname, struct porttab *ports,
     delwin(list.win);
     del_panel(list.borderpanel);
     delwin(list.borderwin);
+    del_panel(statpanel);
+    delwin(statwin);
     unmark_facility(TCPUDPIDFILE, ifname);
     update_panels();
     doupdate();
     destroyportlist(&list);
-    destroyfraglist();
-    destroy_isdn_table(&isdntable);
+    pkt_cleanup();
     strcpy(current_logfile, "");
 }
 
@@ -901,8 +1048,8 @@ void portdlg(unsigned int *port_min, int *port_max, int *aborted, int mode)
     box(bw, ACS_VLINE, ACS_HLINE);
 
     wattrset(win, DLGTEXTATTR);
-    colorwin(win);
-    stdwinset(win);
+    tx_colorwin(win);
+    tx_stdwinset(win);
     wtimeout(win, -1);
 
     mvwprintw(win, 1, 1, "Port numbers below 1024 are reserved for");
@@ -917,14 +1064,14 @@ void portdlg(unsigned int *port_min, int *port_max, int *aborted, int mode)
     tabkeyhelp(win);
     stdkeyhelp(win);
 
-    initfields(&list, 1, 20, (LINES - 14) / 2 + 10, (COLS - 50) / 2 - 8);
-    wattrset(list.fieldwin, DLGTEXTATTR);
+    tx_initfields(&list, 1, 20, (LINES - 14) / 2 + 10, (COLS - 50) / 2 - 8,
+        DLGTEXTATTR, FIELDATTR);
     mvwprintw(list.fieldwin, 0, 6, "to");
 
-    addfield(&list, 5, 0, 0, "");
-    addfield(&list, 5, 0, 9, "");
+    tx_addfield(&list, 5, 0, 0, "");
+    tx_addfield(&list, 5, 0, 9, "");
 
-    fillfields(&list, aborted);
+    tx_fillfields(&list, aborted);
 
     if (!(*aborted)) {
 	*port_min = atoi(list.list->buf);
@@ -934,7 +1081,7 @@ void portdlg(unsigned int *port_min, int *port_max, int *aborted, int mode)
     delwin(bw);
     del_panel(panel);
     delwin(win);
-    destroyfields(&list);
+    tx_destroyfields(&list);
 }
 
 void saveportlist(struct porttab *table)
@@ -947,7 +1094,7 @@ void saveportlist(struct porttab *table)
     fd = open(PORTFILE, O_WRONLY | O_TRUNC | O_CREAT, S_IRUSR | S_IWUSR);
 
     if (fd < 0) {
-	errbox("Unable to open port list file", ANYKEY_MSG, &resp);
+	tx_errbox("Unable to open port list file", ANYKEY_MSG, &resp);
 	return;
     }
     while (ptmp != NULL) {
@@ -955,7 +1102,7 @@ void saveportlist(struct porttab *table)
 	bw = write(fd, &(ptmp->port_max), sizeof(unsigned int));
 
 	if (bw < 0) {
-	    errbox("Unable to write port/range entry", ANYKEY_MSG, &resp);
+	    tx_errbox("Unable to write port/range entry", ANYKEY_MSG, &resp);
 	    destroyporttab(table);
 	    close(fd);
 	    return;
@@ -992,7 +1139,7 @@ void addmoreports(struct porttab **table)
 
     if (!aborted) {
 	if (dup_portentry(*table, port_min, port_max))
-	    errbox("Duplicate port/range entry", ANYKEY_MSG, &resp);
+	    tx_errbox("Duplicate port/range entry", ANYKEY_MSG, &resp);
 	else {
 	    ptmp = malloc(sizeof(struct porttab));
 
@@ -1033,7 +1180,7 @@ void loadaddports(struct porttab **table)
 	br = read(fd, &(ptemp->port_max), sizeof(unsigned int));
 
 	if (br < 0) {
-	    errbox("Error reading port list", ANYKEY_MSG, &resp);
+	    tx_errbox("Error reading port list", ANYKEY_MSG, &resp);
 	    close(fd);
 	    destroyporttab(*table);
 	    return;
@@ -1082,117 +1229,45 @@ void displayports(struct porttab **table, WINDOW * win)
 }
 
 void operate_portselect(struct porttab **table, struct porttab **node,
-			WINDOW * win, int *aborted)
+			int *aborted)
 {
     int ch = 0;
-    int i = 0;
+    struct scroll_list list;
+    char listtext[20];
 
-    int endloop = 0;
+    tx_init_listbox(&list, 25, 22, (COLS - 25) / 2, (LINES - 22) / 2,
+        STDATTR, BOXATTR, BARSTDATTR, HIGHATTR);
 
-    *node = *table;
+    tx_set_listbox_title(&list, "Select Port/Range", 1);
+    
+    *node = *table;    
+    while (*node != NULL) {
+        snprintf(listtext, 20, "%d to %d", (*node)->port_min, (*node)->port_max);
+        tx_add_list_entry(&list, (char *) *node, listtext);
+        *node = (*node)->next_entry;
+    }
+    
+    tx_show_listbox(&list);
+    tx_operate_listbox(&list, &ch, aborted);
+    
+    if (!(*aborted))
+        *node = (struct porttab *) list.textptr->nodeptr;    
 
-    do {
-	wattrset(win, PTRATTR);
-	wmove(win, i, 1);
-	waddch(win, ACS_RARROW);
-	ch = wgetch(win);
-	wmove(win, i, 1);
-	waddch(win, ' ');
-	wattrset(win, STDATTR);
-
-	switch (ch) {
-	case KEY_DOWN:
-	    if ((*node)->next_entry != NULL) {
-		*node = (*node)->next_entry;
-
-		if (i < 17)
-		    i++;
-		else {
-		    wscrl(win, 1);
-		    scrollok(win, 0);
-		    wmove(win, 17, 0);
-		    wprintw(win, "%22c", ' ');
-		    scrollok(win, 1);
-		    wmove(win, 17, 2);
-		    displayportentry((*node), win);
-		}
-	    }
-	    break;
-	case KEY_UP:
-	    if ((*node)->prev_entry != NULL) {
-		*node = (*node)->prev_entry;
-
-		if (i > 0)
-		    i--;
-		else {
-		    wscrl(win, -1);
-		    wmove(win, 0, 0);
-		    wprintw(win, "%22c", ' ');
-		    wmove(win, 0, 2);
-		    displayportentry((*node), win);
-		}
-	    }
-	    break;
-	case 13:
-	    endloop = 1;
-	    *aborted = 0;
-	    break;
-	case 12:
-	case 'l':
-	case 'L':
-	    refresh_screen();
-	    break;
-	case 27:
-	case 24:
-	case 'x':
-	case 'X':
-	case 'q':
-	case 'Q':
-	    endloop = 1;
-	    *aborted = 1;
-	    break;
-	}
-    } while (!endloop);
+    tx_close_listbox(&list);
+    tx_destroy_list(&list);
 }
 
 void selectport(struct porttab **table, struct porttab **node,
 		int *aborted)
 {
-    WINDOW *bw;
-    PANEL *bp;
-    WINDOW *win;
-    PANEL *panel;
     int resp;
-
+    
     if (*table == NULL) {
-	errbox("No custom ports", ANYKEY_MSG, &resp);
+	tx_errbox("No custom ports", ANYKEY_MSG, &resp);
 	return;
     }
-    bw = newwin(20, 24, (LINES - 20) / 2, (COLS - 24) / 2 - 5);
-    bp = new_panel(bw);
-    wattrset(bw, BOXATTR);
-    box(bw, ACS_VLINE, ACS_HLINE);
-    wmove(bw, 0, 1);
-    wprintw(bw, " Select port entry ");
 
-    win = newwin(18, 22, (LINES - 18) / 2, (COLS - 22) / 2 - 5);
-    panel = new_panel(win);
-    stdwinset(win);
-    wattrset(win, STDATTR);
-    colorwin(win);
-
-    listkeyhelp();
-
-    displayports(table, win);
-    operate_portselect(table, node, win, aborted);
-
-    del_panel(panel);
-    delwin(win);
-    del_panel(bp);
-    delwin(bw);
-
-    update_panels();
-    doupdate();
+    operate_portselect(table, node, aborted);
 }
 
 void delport(struct porttab **table, struct porttab *ptmp)
