@@ -6,6 +6,12 @@ Copyright (c) Gerard Paul Java 1997
 
 ***/
 
+struct serv_spans {
+        int spanbr_in;
+        int spanbr_out;
+        int spanbr;
+};
+    
 struct portlistent {
     unsigned int port;
     unsigned int protocol;
@@ -17,6 +23,9 @@ struct portlistent {
     unsigned long long ibcount;
     unsigned long long ocount;
     unsigned long long obcount;
+    time_t starttime;
+    time_t proto_starttime;
+    struct serv_spans spans;    
     struct portlistent *prev_entry;
     struct portlistent *next_entry;
 };
@@ -26,6 +35,9 @@ struct portlist {
     struct portlistent *tail;
     struct portlistent *firstvisible;
     struct portlistent *lastvisible;
+    struct portlistent *barptr;
+    int imaxy;
+    unsigned int baridx;
     unsigned int count;
     unsigned long bcount;
     WINDOW *win;
