@@ -17,15 +17,16 @@ details.
    
 ***/
 
+#include <winops.h>
+#include "options.h"
 #include "tcptable.h"
-#include "stdwinset.h"
 #include "deskman.h"
 #include "attrs.h"
 #include "log.h"
 #include "revname.h"
 #include "rvnamed.h"
 
-#define MSGSTRING_MAX	255
+#define MSGSTRING_MAX	320
 
 unsigned int bmaxy = 0;
 unsigned int imaxy = 0;
@@ -33,8 +34,6 @@ unsigned int imaxy = 0;
 void convmacaddr(char *addr, char *result);
 void writetcplog(int logging, FILE * fd, struct tcptableent *entry,
 		 unsigned int pktlen, int mac, char *message);
-void write_timeout_log(int logging, FILE * logfile,
-		       struct tcptableent *tcpnode);
 
 void setlabels(WINDOW * win, int mode)
 {
@@ -111,12 +110,12 @@ void init_tcp_table(struct tcptable *table)
     table->tcppanel = new_panel(table->tcpscreen);
     table->closedentries = table->closedtail = NULL;
     wattrset(table->tcpscreen, BOXATTR);
-    colorwin(table->tcpscreen);
+    tx_colorwin(table->tcpscreen);
     table->lastpos = 0;
     table->count = 0;
 
     wtimeout(table->tcpscreen, -1);
-    stdwinset(table->tcpscreen);
+    tx_stdwinset(table->tcpscreen);
     print_tcp_num_entries(table);
 
     /* 
@@ -127,6 +126,8 @@ void init_tcp_table(struct tcptable *table)
 	table->hash_table[i] = NULL;
 	table->hash_tails[i] = NULL;
     }
+    table->barptr = NULL;
+    table->baridx = 0;
 }
 
 /*
@@ -380,7 +381,21 @@ struct tcptableent *addentry(struct tcptable *table,
     new_entry->finack = new_entry->oth_connection->finack = 0;
     new_entry->finsent = new_entry->oth_connection->finsent = 0;
     new_entry->partial = new_entry->oth_connection->partial = 0;
-
+    new_entry->spanbr = new_entry->oth_connection->spanbr = 0;
+    new_entry->conn_starttime = new_entry->oth_connection->conn_starttime = time(NULL);
+    
+    /*
+     * Mark flow rate start time and byte counter for flow computation
+     * if the highlight bar is on either flow of the new connection.
+     */
+    if (table->barptr == new_entry) {
+        new_entry->starttime = time(NULL);
+        new_entry->spanbr = 0;
+    } else if (table->barptr == new_entry->oth_connection) {
+        new_entry->oth_connection->starttime = time(NULL); 
+        new_entry->oth_connection->spanbr = 0;
+    }   
+        
     /*
      * Add entries to hash table
      */
@@ -425,11 +440,59 @@ void addtoclosedlist(struct tcptable *table, struct tcptableent *entry,
 
 }
 
+char *tcplog_flowrate_msg(struct tcptableent *entry, struct OPTIONS *opts)
+{
+    char rateunit[10];
+    float rate = 0;
+    static char message[60];
+    time_t interval;
+
+    interval = time(NULL) / (float) (entry->conn_starttime - time(NULL));
+    
+    if (opts->actmode == KBITS) {
+         strcpy(rateunit, "kbits/s");
+         
+         if (interval > 0)
+             rate = (float) (entry->bcount * 8 / 1000) / (float) interval;
+         else
+             rate = 0.00;
+    } else {
+        strcpy(rateunit, "kbytes/s");
+        
+        if (interval > 0)
+            rate = (float) (entry->bcount / 1024) / (float) interval;
+        else
+            rate = 0.00;
+    }
+   
+    snprintf(message, 60, "avg flow rate %.2f %s", rate, rateunit);
+    return message;
+}
+
+void write_timeout_log(int logging, FILE * logfile,
+		       struct tcptableent *tcpnode, struct OPTIONS *opts)
+{
+    char msgstring[MSGSTRING_MAX];
+
+    if (logging) {
+	snprintf(msgstring, MSGSTRING_MAX,
+		"TCP; Connection %s:%s to %s:%s timed out, %lu packets, %lu bytes, %s; opposite direction %lu packets, %lu bytes, %s",
+		tcpnode->s_fqdn, tcpnode->s_sname,
+		tcpnode->d_fqdn, tcpnode->d_sname,
+		tcpnode->pcount, tcpnode->bcount,
+		tcplog_flowrate_msg(tcpnode, opts),
+		tcpnode->oth_connection->pcount,
+		tcpnode->oth_connection->bcount,
+		tcplog_flowrate_msg(tcpnode->oth_connection, opts));
+	writelog(logging, logfile, msgstring);
+    }
+}
+
 struct tcptableent *in_table(struct tcptable *table, unsigned long saddr,
 			     unsigned long daddr, unsigned int sport,
 			     unsigned int dport, char *ifname,
-			     unsigned int tm, int logging, FILE * logfile,
-			     int *nomem)
+			     int logging, FILE * logfile,
+			     int *nomem, struct OPTIONS *opts)
 {
     struct tcp_hashentry *hashptr;
     unsigned int hp;
@@ -437,6 +500,12 @@ struct tcptableent *in_table(struct tcptable *table, unsigned long saddr,
     int hastimeouts = 0;
 
     time_t now;
+    time_t timeout;
+    
+    if (opts != NULL)
+        timeout = opts->timeout;
+    else
+        timeout = 0;
 
     if (table->head == NULL) {
 	return 0;
@@ -463,7 +532,7 @@ struct tcptableent *in_table(struct tcptable *table, unsigned long saddr,
 	 * find any closed ones.
 	 */
 
-	if ((tm > 0) && ((now - hashptr->tcpnode->lastupdate) / 60 > tm)
+	if ((timeout > 0) && ((now - hashptr->tcpnode->lastupdate) / 60 > timeout)
 	    && (!(hashptr->tcpnode->inclosed))) {
 	    hashptr->tcpnode->timedout = 1;
 	    hashptr->tcpnode->oth_connection->timedout = 1;
@@ -472,7 +541,7 @@ struct tcptableent *in_table(struct tcptable *table, unsigned long saddr,
 		hastimeouts = 1;
 
 	    if (logging)
-		write_timeout_log(logging, logfile, hashptr->tcpnode);
+		write_timeout_log(logging, logfile, hashptr->tcpnode, opts);
 	}
 	hashptr = hashptr->next_entry;
     }
@@ -491,6 +560,7 @@ struct tcptableent *in_table(struct tcptable *table, unsigned long saddr,
     }
 }
 
+             
 /*
  * Update the TCP status record should an applicable packet arrive.
  */
@@ -499,7 +569,8 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 		 struct tcphdr *transpacket, char *packet, int linkproto,
 		 unsigned long packetlength,
 		 unsigned int bcount, unsigned int fragofs, int logging,
-		 int mac, int *revlook, int rvnfd, FILE * logfile, int *nomem)
+		 int *revlook, int rvnfd, 
+		 struct OPTIONS *opts, FILE * logfile, int *nomem)
 {
     char msgstring[MSGSTRING_MAX];
     char newmacaddr[15];
@@ -519,8 +590,9 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
     tableentry->pcount++;
     tableentry->bcount += bcount;
     tableentry->psize = packetlength;
+    tableentry->spanbr += bcount;
 
-    if (mac) {
+    if (opts->mac) {
         bzero(newmacaddr, 15);
     
     	if ((linkproto == LINK_ETHERNET) || (linkproto == LINK_PLIP)) {
@@ -529,6 +601,8 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	} else if (linkproto == LINK_FDDI) {
 	    convmacaddr(((struct fddihdr *) packet)->saddr,
 			newmacaddr);
+        } else if (linkproto == LINK_TR) {
+            convmacaddr(((struct trh_hdr *) packet)->saddr, newmacaddr);
         }
         
         if (tableentry->smacaddr[0] != '\0') {
@@ -590,7 +664,7 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 
 	    if (logging) {
 		writetcplog(logging, logfile, tableentry,
-			    tableentry->psize, mac, "FIN acknowleged");
+			    tableentry->psize, opts->mac, "FIN acknowleged");
 	    }
 	}
     }
@@ -635,11 +709,11 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	    tableentry->finack = ntohl(transpacket->ack_seq);
 	}
 	if (logging) {
-	    sprintf(msgstring, "FIN sent; %lu packets, %lu bytes",
-		    tableentry->pcount, tableentry->bcount);
+	    sprintf(msgstring, "FIN sent; %lu packets, %lu bytes, %s",
+		    tableentry->pcount, tableentry->bcount, tcplog_flowrate_msg(tableentry, opts));
 
 	    writetcplog(logging, logfile, tableentry, tableentry->psize,
-			mac, msgstring);
+			opts->mac, msgstring);
 	}
     }
     if (transpacket->rst) {
@@ -648,13 +722,13 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	    addtoclosedlist(table, tableentry, nomem);
 
 	if (logging) {
-	    sprintf(msgstring,
-		    "Connection reset; %lu packets, %lu bytes; opposite direction %lu packets, %lu bytes",
-		    tableentry->pcount, tableentry->bcount,
+	    snprintf(msgstring, MSGSTRING_MAX,
+		    "Connection reset; %lu packets, %lu bytes, %s; opposite direction %lu packets, %lu bytes; %s",
+		    tableentry->pcount, tableentry->bcount, tcplog_flowrate_msg(tableentry, opts),
 		    tableentry->oth_connection->pcount,
-		    tableentry->oth_connection->bcount);
+		    tableentry->oth_connection->bcount, tcplog_flowrate_msg(tableentry->oth_connection, opts));
 	    writetcplog(logging, logfile, tableentry, tableentry->psize,
-			mac, msgstring);
+			opts->mac, msgstring);
 	}
     }
     if (transpacket->psh)
@@ -718,7 +792,21 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
     char stat[7] = "";
     unsigned int target_row;
     char sp_buf[MSGSTRING_MAX];
-
+    int normalattr;
+    int highattr;
+    
+    /*
+     * Set appropriate attributes for this entry
+     */
+     
+    if (table->barptr == tableentry) {
+        normalattr = BARSTDATTR;
+        highattr = BARHIGHATTR;
+    } else {
+        normalattr = STDATTR;
+        highattr = HIGHATTR;
+    }
+    
     if ((tableentry->index < screen_idx) ||
 	(tableentry->index > screen_idx + (table->imaxy - 1)))
 	return;
@@ -744,14 +832,17 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
 
     /* proceed with the actual entry */
 
+    wattrset(table->tcpscreen, normalattr);
+    sprintf(sp_buf, "%%%dc", COLS - 5);
+    mvwprintw(table->tcpscreen, target_row, 2, sp_buf, ' ');
+    
     sprintf(sp_buf, "%%.%ds:%%.%ds", 32 * COLS / 80, 10);
-    wattrset(table->tcpscreen, STDATTR);
 
     wmove(table->tcpscreen, target_row, 1);
     wprintw(table->tcpscreen, sp_buf, tableentry->s_fqdn,
 	    tableentry->s_sname);
 
-    wattrset(table->tcpscreen, HIGHATTR);
+    wattrset(table->tcpscreen, highattr);
 
     /* 
      * Print packet and byte counts or window size and packet size, depending
@@ -783,7 +874,7 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
 	wprintw(table->tcpscreen, "%9u  ", tableentry->win);
     }
 
-    wattrset(table->tcpscreen, STDATTR);
+    wattrset(table->tcpscreen, normalattr);
 
     if (tableentry->finsent == 1)
 	strcpy(stat, "DONE  ");
@@ -828,22 +919,17 @@ void printentry(struct tcptable *table, struct tcptableent *tableentry,
 void refreshtcpwin(struct tcptable *table, unsigned int idx, int mode)
 {
     struct tcptableent *ptmp;
-    int endloop = 0;
 
     setlabels(table->borderwin, mode);
-    colorwin(table->tcpscreen);
+    wattrset(table->tcpscreen, STDATTR);
+    tx_colorwin(table->tcpscreen);
     ptmp = table->firstvisible;
 
-    if (ptmp != NULL) {
-	do {
-	    printentry(table, ptmp, idx, mode);
-	    ptmp = ptmp->next_entry;
-	    if (ptmp == NULL)
-		endloop = 1;
-	    else if (ptmp->prev_entry == table->lastvisible)
-		endloop = 1;
-	} while (!endloop);
+    while ((ptmp != NULL) && (ptmp->prev_entry != table->lastvisible)) {
+	printentry(table, ptmp, idx, mode);
+	ptmp = ptmp->next_entry;
     }
+
     wmove(table->borderwin, table->bmaxy - 1, 1);
 
     print_tcp_num_entries(table);
@@ -956,9 +1042,9 @@ void destroy_tcp_entry(struct tcptable *table, struct tcptableent *ptmp)
  * entries.
  */
 
-void flushclosedentries(struct tcptable *table, time_t tm,
+void flushclosedentries(struct tcptable *table,
 			unsigned long *screen_idx,
-			int logging, FILE * logfile)
+			int logging, FILE * logfile, struct OPTIONS *opts)
 {
     struct tcptableent *ptmp = table->head;
     struct tcptableent *ctmp = NULL;
@@ -970,14 +1056,14 @@ void flushclosedentries(struct tcptable *table, time_t tm,
 	now = time(NULL);
 	lastupdated = (now - ptmp->lastupdate) / 60;
 
-	if ((ptmp->inclosed) || (lastupdated > tm)) {
+	if ((ptmp->inclosed) || (lastupdated > opts->timeout)) {
 	    ctmp = ptmp;
 	    /*
 	     * Mark and flush timed out TCP entries.
 	     */
-	    if (lastupdated > tm) {
+	    if (lastupdated > opts->timeout) {
 		if ((!(ptmp->timedout)) && (!(ptmp->inclosed))) {
-		    write_timeout_log(logging, logfile, ptmp);
+		    write_timeout_log(logging, logfile, ptmp, opts);
 		    ptmp->timedout = ptmp->oth_connection->timedout = 1;
 		}
 	    }
@@ -986,18 +1072,49 @@ void flushclosedentries(struct tcptable *table, time_t tm,
 	     * Advance to next entry and destroy target entry.
 	     */
 	    ptmp = ptmp->next_entry;
+	    
+	    /*
+	     * If the targeted entry is highlighted, and the next entry is
+	     * not NULL (we're still in the list) we move the bar pointer to
+	     * the next entry otherwise we move it to the previous entry.
+	     */
+	    if (ptmp != NULL) {
+	        if (table->barptr == ctmp) {
+	            table->barptr = ptmp;
+	        }
+	    } else {
+	        if (table->barptr == ctmp) {
+	            table->barptr = table->barptr->prev_entry;
+	        }
+	    }
+	    
+	    /*
+	     * Do the dirty deed
+	     */ 
 	    del_tcp_hash_node(table, ctmp);
 	    destroy_tcp_entry(table, ctmp);
 	    
+	    /*
+	     * Adjust screen index if the deleted entry was "above"
+	     * the screen.
+	     */
 	    if (idx < *screen_idx)
 	        (*screen_idx)--;
 	} else {
+	    /*
+	     * Set the first visible pointer once the index matches
+	     * the screen index.
+	     */
 	    if (idx == *screen_idx)
 		table->firstvisible = ptmp;
 
+            /*
+             * Keep setting the last visible pointer until the scan
+             * index "leaves" the screen
+             */
 	    if (idx <= (*screen_idx) + (table->imaxy - 1))
 		table->lastvisible = ptmp;
-
+	    
 	    ptmp->index = idx;
 	    idx++;
 	    ptmp = ptmp->next_entry;
@@ -1015,7 +1132,6 @@ void flushclosedentries(struct tcptable *table, time_t tm,
      */
 
     if (table->head != NULL) {
-
 	/*
 	 * Point screen index to the last table entry if the tail entry is
 	 * "above" the screen index.  Set the firstvisible pointer to that
@@ -1030,12 +1146,18 @@ void flushclosedentries(struct tcptable *table, time_t tm,
 	 * Move the screen index and firstvisible entry up until the tail
 	 * hits the bottom of the window (tail is at screen index plus
 	 * screen length minus 1) or the firstvisible pointer hits the
-	 * head of the table.
+	 * head of the table.  The highlight bar should "go along" with
+	 * the shifting.
 	 */
 	while ((table->tail->index < *screen_idx + table->imaxy - 1) &&
 	       (table->firstvisible->prev_entry != NULL)) {
 	    table->firstvisible = table->firstvisible->prev_entry;
 	    (*screen_idx)--;
 	}
+	
+	/*
+	 * Set the bar position index once everything's done.
+	 */
+        table->baridx = table->barptr->index - *screen_idx + 1;
     }
 }
